/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             139 Main Street                             */
/*             Cambridge, MA 02142, USA                    */
/*             Version 4.0        1998                     */
/*             Author: Daniel Griscom                      */
/*             		   Suitable Systems                    */
/*             		   griscom@suitable.com                */
/*             		   www.suitable.com                    */
/***********************************************************/

/* File: arg.c
	This code deals with parsing arguments for a command.
	*/

#include "bhand.h"
#include "param.h"
#include "line.h"
#include "cmd.h"
#include "temp.h"
#include "hp.h"
#include "arg.h"
#include "utils.h"

/* ====================== Local prototypes ======================== */

/* curMotorFWrite writes a given parameter for the current motor. */
void curMotorFWrite(int whichParam, unsigned int theValue);

/* curMotorFRead reads a given parameter for the current motor. */
unsigned int curMotorFRead(int whichParam);

/* ====================== Global vars ======================== */

const argStruct FArgs[] = {     /* List of finger [motor] parameters */
	/* Fields:
		name:		String; parameter name
		visible:	Flag: true iff parameter should be listed by "FLIST" command
		readFunc:	Function pointer for reading value (0 if not allowed)
		writeFunc:	Function pointer for writing value (0 if not allowed)
		firstParam:	Value sent to readFunc or writeFunc as first param (second param for
							writeFunc is value to be written)
		help:		String; help string printed out by "FLIST" command
		*/
	/* name,  visible,   readFunc,       writeFunc,      firstParam,      help                                        */
	{ "S",		true,   hpStatusRead,   0,              0,              "Current motor state" },
	{ "P",		true,	positionRead,	0,				0,				"Current motor position" },
	{ "SG",		true,	strainGaugeRead,0,				0,				"Current strain gauge reading" },
	{ "MOV",	true,   curMotorFRead,	curMotorFWrite,	kFParamMOV,		"Maximum open velocity" },
	{ "MCV",	true,   curMotorFRead,	curMotorFWrite,	kFParamMCV,		"Maximum close velocity" },
	{ "MSG",	true,   curMotorFRead,	curMotorFWrite,	kFParamMSG,		"Maximum allowable finger closing strain" },
	{ "DS",		true,   curMotorFRead,	curMotorFWrite,	kFParamDS,      "Default step size for IO/IC" },
	{ "DP",		true,   curMotorFRead,	curMotorFWrite,	kFParamDP,      "Default position for M" },
	{ "LCV",	true,	curMotorFRead,	curMotorFWrite,	kFParamLCV,		"Flag: loop control block contains velocity" },
	{ "LCVC",	true,	curMotorFRead,	curMotorFWrite,	kFParamLCVC,	"LCV is multiplied by this to get control velocity" },
	{ "LCPG",	true,	curMotorFRead,	curMotorFWrite,	kFParamLCPG,	"Flag: loop control block contains proportional gain" },
	{ "LFV",	true,	curMotorFRead,	curMotorFWrite,	kFParamLFV,		"Flag: loop feedback block contains velocity" },
	{ "LFVC",	true,	curMotorFRead,	curMotorFWrite,	kFParamLFVC,	"Velocity is divided by this to get LFV" },
	{ "LFS",	true,	curMotorFRead,	curMotorFWrite,	kFParamLFS,		"Flag: loop control block contains strain" },
	{ "LFAP",	true,	curMotorFRead,	curMotorFWrite,	kFParamLFAP,	"Flag: loop control block contains absolute position" },
	{ "LFDP",	true,	curMotorFRead,	curMotorFWrite,	kFParamLFDP,	"Flag: loop control block contains delta position" },
	{ "LFDPC",	true,	curMotorFRead,	curMotorFWrite,	kFParamLFDPC,	"Delta position is divided by this to get LFDP" },
	{ "FPG",	false,  curMotorFRead,	curMotorFWrite,	kFParamFPG,		"Proportional gain for motor filter" },
	{ "FIP",	false,  curMotorFRead,	curMotorFWrite,	kFParamFIP,		"Integral pole for motor filter" },
	{ "FDZ",	false,  curMotorFRead,	curMotorFWrite,	kFParamFDZ,		"Derivative zero for motor filter" },
	{ "ACCEL",	false,	curMotorFRead,	curMotorFWrite,	kFParamACCEL,	"Acceleration" },
	{ "MPE",	false,	curMotorFRead,	curMotorFWrite,	kFParamMPE,		"Max dist from desired pos before error" },
	{ "TSTOP",	false,  curMotorFRead,	curMotorFWrite,	kFParamTSTOP,	"Time in ms before motor considered stopped" },
	{ "HOLD",	false,	curMotorFRead,	curMotorFWrite,	kFParamHOLD,	"Flag: motor should hold position" },
	{ "SGFLIP",	false,	curMotorFRead,	curMotorFWrite,	kFParamSGFLIP,	"Flag: strain gauge value reversed" },
	{ "EN",		false,	curMotorFRead,	curMotorFWrite,	kFParamEN,		"Flag: motor is by default selected" },
	{ "",		0,      0,              0,              0,              "" },
	};

const argStruct PArgs[] = {     /* List of global parameters */
	/* name,   visible,  readFunc,		 writeFunc,      firstParam,     help */
	{ "TEMP",	true,   tempRead,		0,              0,              "Temperature, in tenths of a degree C" },
	{ "BAUD",	false,	paramGRead,		paramGWrite,	kGParamBAUD,	"Baud rate divided by 100" },
	{ "",		0,      0,          	0,              0,              "" },
	};

/* ====================== Static vars ======================== */

static int count;                               /* Number of arguments found in last parse. */
static unsigned int argValues[maxWords + 1];    /* Values of arguments. 1-based. */
static argStruct *argParams[maxWords + 1];      /* Argument structures. 1-based. */

/* ====================== Functions ======================== */

/* argIntListParse parses the argument list as a list of integers, which can
	then be retrieved by argIntListGet(). Returns error (if any). */
int argIntListParse(void)
{
	long value;
	int i;

	count = lineWordCount() - 1;
	if (count > maxWords)
		return errTooManyArgs;
	for (i = 1; i <= count; i++) {
		value = atol(lineWordGet(i));
		if ((value > 0xFFFFL) || (value < 0L))
			return errBadValue;		/* !!!! Test this !!!! */
		argValues[i] = (unsigned int)value;
	}
	return errNone;
}

/* argCount returns the number of arguments (or argument pairs in the case
	of argParamPairListParse). */
int argCount(void)
{
	return count;
}

/* argIntListGet returns a given integer argument. whichArg is 1-based. */
unsigned int argIntListGet(int whichArg)
{
	return argValues[whichArg];
}

/* argParamListParse takes an argument list structure. It parses the argument
	list, searching for each parameter name. Returns error (if any). */
int argParamListParse(argStruct *argArray)
{
	int i;
	argStruct *asp;
	char *s;

	count = lineWordCount() - 1;
	if (count > maxWords)
		return errTooManyArgs;

	for (i = 1; i <= count; i++) {
		asp = argArray;
		s = lineWordGet(i);
		copReset();
		while (true) {
			/* If ran out of elements in argArray, error. */
			if (asp->name[0] == '\0')
				return errBadParam;		/* !!!!! Test this !!!!! */
			/* If found parameter, remember it and stop looking */
			if (strcmp(asp->name, s) == 0) {
				argParams[i] = asp;
				break;
			}
			asp++;  /* Step to next element in argArray */
		}
	}
	return errNone;         /* Found each param: no error. */
}

/* argParamListRead uses the readFunc pointer to return the value of the given
	parameter. Note that the readFunc pointer may take into account the
	current motor num to decide what value to return. */
unsigned int argParamListRead(int whichArg)
{
	unsigned int (*readFunc)(int), value;
	int firstParam;

	readFunc = argParams[whichArg]->readFunc;
	firstParam = argParams[whichArg]->firstParam;
	value = (readFunc)(firstParam);
	return value;
}

/* argParamPairListParse takes an argument list structure. It parse the argument
	list, searching for parameter names and converting values. Returns
	error (if any). */
int argParamPairListParse(argStruct *argArray)
{
	int i;
	argStruct *asp;
	char *s;
	long value;
	int err;

	count = lineWordCount() - 1;
	if (count % 2)
		return errTooManyArgs;		/* !!!! Test !!!! */
	count = count / 2;
	if (count > maxWords)
		return errTooManyArgs;
	
	err = errNone;

	for (i = 1; i <= count; i++) {
		copReset();
		asp = argArray;
		s = lineWordGet(i * 2 - 1);
		while (true) {
			/* If ran out of elements in argArray, error. */
			if (asp->name[0] == '\0')
				return errBadParam;	/* !!!! Test !!!! */
			/* If found parameter, remember it and stop looking */
			if (strcmp(asp->name, s) == 0) {
				argParams[i] = asp;
				if (asp->writeFunc == 0)
					err |= errReadOnly; 
				break;
			}
			asp++;  /* Step to next element in argArray */
		}
		value = atol(lineWordGet(i * 2));
		if ((value > 0xFFFFL) || (value < 0L))
			return errBadValue;		/* !!!! Test !!!! */
		argValues[i] = (unsigned int)value;
	}
	return errNone;         /* Found each param: no error. */
}


/* argParamPairListWrite uses the writeFunc pointer and the value found in the
	argument list to write the given parameter. Note that the writeFunc
	pointer may take into account the current motor num to decide what
	parameter to write. */
void argParamPairListWrite(int whichArg)
{
	void (*writeFunc)(int, unsigned int);
	int firstParam;

	writeFunc = argParams[whichArg]->writeFunc;
	firstParam = argParams[whichArg]->firstParam;
	(writeFunc)(firstParam, argValues[whichArg]);
}



/* argPrintList prints out the arguments in the given argument structure array,
	one per line. If showAll is false then only shows visible parameters. */
void argPrintList(argStruct *argArray, int showAll)
{
	argStruct *asp;
	
	for (asp = argArray; asp->name[0] != '\0'; asp++)
		if (asp->visible || showAll)
			printf("\n%s: %s", asp->name, asp->help);
}

/* argFPrintValues prints out the values of the arguments in the finger
	argument structure array. If showAll is false then only shows
	visible parameters. Shows one value for each selected motor. */
void argFPrintValues(int showAll, motorFlags motors)
{
	argStruct *asp;
	unsigned int (*readFunc)(int), value;
	int motor;
	
	for (asp = FArgs; asp->name[0] != '\0'; asp++) {
		D2(printf("\n asp = %4.4x; looking at %s...", asp, asp->name));
		if (asp->visible || showAll) {
			printf("\n");
			readFunc = asp->readFunc;
			for (motor = 1; motor <= 4; motor++)
				if (motors[motor]) {
					hpSetMotor(motor);
					value = (readFunc)(asp->firstParam);
					printf("%d ", value);
				}
		}
	}
}

/* argPPrintValues prints out the values of the arguments in the
	global argument structure array. If showAll is false then only
	shows visible parameters. */
void argPPrintValues(int showAll)
{
	argStruct *asp;
	unsigned int (*readFunc)(int), value;
	
	for (asp = PArgs; asp->name[0] != '\0'; asp++)
		if (asp->visible || showAll) {
			readFunc = asp->readFunc;
			value = (readFunc)(asp->firstParam);
			printf("\n%d ", value);
		}
}

/* curMotorFWrite writes a given parameter for the current motor. */
void curMotorFWrite(int whichParam, unsigned int theValue)
{
	paramFWrite(hpGetMotor(), whichParam, theValue);
}

/* curMotorFRead reads a given parameter for the current motor. */
unsigned int curMotorFRead(int whichParam)
{
	return paramFRead(hpGetMotor(), whichParam);
}

