/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             139 Main Street                             */
/*             Cambridge, MA 02142, USA                    */
/*             Version 4.0        1998                     */
/*             Author: Daniel Griscom                      */
/*             		   Suitable Systems                    */
/*             		   griscom@suitable.com                */
/*             		   www.suitable.com                    */
/***********************************************************/

/* File: cmd.h
	This file holds all the command functions. The functions are called
	by main() when one matches the first word of the command line.
	The command structure is also defined here. */

#ifndef _cmd_h
#define _cmd_h

#include "bhand.h"

/* ===================== Types and storage ====================== */

/* CommandType is a structure that defines a command. */
typedef	struct {
	char name[8];	/* Name of command (max 7 chars plus null) */
	int visible;	/* Flag: true if command is visible to customers */
	int isMotor;	/* Flag: true if motor command that can take motor prefixes */
	intFuncVoidPtr func; /* Pointer to command function; returns integer error */
	char help[45];	/* Help string for command (max 44 chars plus null) */
	} CommandType;

extern const CommandType CmdList[];	/* Primary list of commands */

/* ===================== Error #defines ====================== */

#define errNone				(0)			/* No error */
#define errBoardNone		(1 << 0)	/* Couldn't find board */
#define errMotorNone		(1 << 1)	/* Couldn't find motor */
#define errMotorNotInit		(1 << 2)	/* Motor not initialized */
/* Not used...				(1 << 3)							*/
#define errCouldntReach		(1 << 4)	/* Couldn't reach position */
#define errBadCommand		(1 << 5)	/* Didn't recognize command */
#define errBadParam			(1 << 6)	/* Didn't recognize parameter */
#define errBadValue			(1 << 7)	/* Bad value */
#define errReadOnly			(1 << 8)	/* Tried to write read-only parameter */
#define errTimeout			(1 << 9)	/* Timeout (never returned here )*/
#define errTooManyArgs		(1 << 10)	/* Too many arguments */
#define errBadHeader		(1 << 11)	/* Bad loop control block header */
#define errNoMotorPrefix	(1 << 12)	/* Command cannot have motor prefix */

/* ===================== Function prototypes ====================== */

/* CmdMotorInit initializes all/selected motors. */
int CmdMotorInit(void);

/* CmdOpen opens all/selected motors. */
int CmdOpen(void);

/* CmdClose closes all/selected motors. */
int CmdClose(void);

/* CmdIncOpen opens all/selected motors by the given amount, or default. */
int CmdIncOpen(void);

/* CmdIncClose closes all/selected motors by the given amount, or default. */
int CmdIncClose(void);

/* CmdMove moves all/selected motors to the given position. */
int CmdMove(void);

/* CmdTerminate turns off all/selected motor power. */
int CmdTerminate(void);

/* CmdHome moves all/selected motors to their home position. */
int CmdHome(void);

/* CmdFingerSet sets the given motor parameters. */
int CmdFingerSet(void);

/* CmdFingerGet outputs the given motor parameters. */
int CmdFingerGet(void);

/* CmdFingerLoad loads parameters for all/selected motors from the EEPROM. */
int CmdFingerLoad(void);

/* CmdFingerSave saves parameters for all/selected motors to the EEPROM. */
int CmdFingerSave(void);

/* CmdFingerDef sets parameters for all/selected motors to their default. */
int CmdFingerDef(void);

/* CmdFingerList lists all public parameter names for motors. */
int CmdFingerList(void);

/* CmdFingerListV lists all public parameter values for all/selected motors. */
int CmdFingerListV(void);

/* CmdFingerListA lists all public and private parameter names for motors. */
int CmdFingerListA(void);

/* CmdFingerListAV lists all public and private parameter values
	for all/selected motors. */
int CmdFingerListAV(void);

/* CmdParamSet sets the given global parameters. */
int CmdParamSet(void);

/* CmdParamGet outputs the given global parameters. */
int CmdParamGet(void);

/* CmdParamLoad loads the global parameters from the EEPROM. */
int CmdParamLoad(void);

/* CmdParamSave saves the global parameters to the EEPROM. */
int CmdParamSave(void);

/* CmdParamDef sets the global parameters to their default values. */
int CmdParamDef(void);

/* CmdParamList lists all public global parameter names. */
int CmdParamList(void);

/* CmdParamListV lists all public global parameter values. */
int CmdParamListV(void);

/* CmdParamList lists all public and private global parameter names. */
int CmdParamListA(void);

/* CmdParamListAV lists all public and private global parameter values. */
int CmdParamListAV(void);

/* CmdReset resets the hand software. */
int CmdReset(void);

/* CmdVersion prints out version information for the hand software. */
int CmdVersion(void);

/* CmdErr prints the meaning of a given error number. If no argument,
	prints meaning of all errors. */
int CmdErr(void);

/* CmdListPublic lists all public commands. */
int CmdListPublic(void);

/* CmdListAll lists all commands. */
int CmdListAll(void);

#endif /* #ifndef _cmd_h */
