/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             139 Main Street                             */
/*             Cambridge, MA 02142, USA                    */
/*             Version 4.0        1998                     */
/*             Author: Daniel Griscom                      */
/*             		   Suitable Systems                    */
/*             		   griscom@suitable.com                */
/*             		   www.suitable.com                    */
/***********************************************************/

/* File: cmd.c
	This file holds all the command functions. The functions are called
	by main() when one matches the first word of the command line.
	The command structure is also defined here. */

#include "bhand.h"
#include "hp.h"
#include "motor.h"
#include "mi.h"
#include "utils.h"
#include "serial.h"
#include "line.h"
#include "arg.h"
#include "param.h"
#include "parse.h"
#include "cmd.h"
#include "loop.h"

/* Table of commands */
const CommandType CmdList[] = {
	/* Fields:
		name:		String; command name
		visible:	Flag: true iff command should be listed by "?" command
		isMotor:	Flag: true if can take motor prefixes
		func:		Function pointer for executing the command
		help:		String; help string printed out bye "?" command
		*/
	/* name[]	visible	isMotor	func			help[]                                       */
	{ "O",		true,	true,	CmdOpen,		"Open"},
	{ "C",		true,	true,	CmdClose,		"Close"},
	{ "IO",		true,	true,	CmdIncOpen,		"Incremental open"},
	{ "IC",		true,	true,	CmdIncClose,	"Incremental close"},
	{ "M",		true,	true,	CmdMove,		"Move to position"},
	{ "T",		true,	true,	CmdTerminate,	"Terminate motor power"},
	{ "HOME",	true,	true,	CmdHome,		"Go to home position"},
	{ "LOOP",	true,	true,	CmdLoop,		"Start loop control"},
	{ "HI",		true,	true,	CmdMotorInit,	"Initialize and go to home"},
	{ "FSET",	true,	true,	CmdFingerSet,	"Set finger parameters"},
	{ "FGET",	true,	true,	CmdFingerGet,	"Get finger parameters"},
	{ "FLOAD",	true,	true,	CmdFingerLoad,	"Load finger parameters"},
	{ "FSAVE",	true,	true,	CmdFingerSave,	"Save finger parameters"},
	{ "FDEF",	true,	true,	CmdFingerDef,	"Set finger parameters to default"},
	{ "FLIST",	true,	false,	CmdFingerList,	"List finger parameters"},
	{ "FLISTV",	true,	true,	CmdFingerListV,	"List finger parameter values"},
	{ "FLISTA",	false,	false,	CmdFingerListA,	"List all finger parameters"},
	{ "FLISTAV",false,	true,	CmdFingerListAV, "List all finger parameter values"},
	{ "PSET",	true,	false,	CmdParamSet,	"Set global parameters"},
	{ "PGET",	true,	false,	CmdParamGet,	"Get global parameters"},
	{ "PLOAD",	true,	false,	CmdParamLoad,	"Load global parameters"},
	{ "PSAVE",	true,	false,	CmdParamSave,	"Save global parameters"},
	{ "PDEF",	true,	false,	CmdParamDef,	"Set global parameters to default"},
	{ "PLIST",	true,	false,	CmdParamList,	"List global parameters"},
	{ "PLISTV",	true,	false,	CmdParamListV,	"List global parameter values"},
	{ "PLISTA",	false,	false,	CmdParamListA,	"List all global parameters"},
	{ "PLISTAV",false,	false,	CmdParamListAV,	"List all global parameter values"},
	{ "RESET",	true,	false,	CmdReset,		"Reset hand software"},
	{ "VERS",	true,	false,	CmdVersion,		"Print hand software information"},
	{ "ERR",	true,	false,	CmdErr,			"Print meaning of error number"},
	/* Note: the next command, since it has lower case characters, can never be run. */
	{ "?<cmd>",	true,	false,	0,				"Print help for a command"},
	{ "?",		true,	false,	0,				"List commands"},
	{ "A?",		false,	false,	CmdListAll,		"List all commands"},
	/* The next command can be run (if "?" alone is entered) but can never be listed. */
	{ "",		false,	false,	CmdListPublic,	""},
	};
	
/* ===================== Internal prototypes ====================== */

void listCommands(int showAll);
int CmdOpenOrClose(int direction);
int CmdIncOpenOrClose(int direction);

/* This assumes that an int called err exists. */
#define checkErr(x)		if ((err = (x)) != errNone)	\
					return err;

#define checkMaxArgs(x)		if (lineWordCount() > (x)+1)	\
					return errTooManyArgs;

/* ===================== Functions ====================== */

/* CmdMotorInit initializes all/selected motors. */
int CmdMotorInit(void)
{
	motorFlags motors;

	checkMaxArgs(0);

	parseGetMotors(motors);

	return MotorInit(motors);
}

/* CmdOpen opens all/selected motors. */
int CmdOpen(void)
{
	return CmdOpenOrClose(kOpen);
}

/* CmdClose closes all/selected motors. */
int CmdClose(void)
{
	return CmdOpenOrClose(kClose);
}

/* CmdIncOpen opens all/selected motors by the given amount, or default. */
int CmdIncOpen(void)
{
	return CmdIncOpenOrClose(kOpen);
}

/* CmdIncClose closes all/selected motors by the given amount, or default. */
int CmdIncClose(void)
{
	return CmdIncOpenOrClose(kClose);
}

/* CmdMove moves all/selected motors to the given position. */
int CmdMove(void)
{
	motorFlags motors;
	PositionType pos[4 + 1];
	int err, motor;

	checkMaxArgs(1);
	
	parseGetMotors(motors);
	
	if (lineWordCount() == 2)
		checkErr(argIntListParse());

	for (motor = 1; motor <= 4; motor++)
		if (lineWordCount() == 2)
			pos[motor].plong = argIntListGet(1);
		else
			pos[motor].plong = paramFRead(motor, kFParamDP);
	
	return motorMoveToPos(motors, pos);
}

/* CmdTerminate turns off all/selected motor power. */
int CmdTerminate(void)
{
	motorFlags motors;

	checkMaxArgs(0);
	
	parseGetMotors(motors);
	
	return motorTerminate(motors);
}

/* CmdHome moves all/selected motors to position 0. */
int CmdHome(void)
{
	motorFlags motors;
	PositionType pos[4 + 1];
	int motor;

	checkMaxArgs(0);
	
	parseGetMotors(motors);

	for (motor = 1; motor <= 4; motor++)
		pos[motor].plong = 0;
	
	return motorMoveToPos(motors, pos);
}

/* CmdFingerSet sets the given motor parameters. */
int CmdFingerSet(void)
{
	int err, motor, argNum;
	motorFlags motors;
	
	checkErr(argParamPairListParse(FArgs));
	
	parseGetMotors(motors);
	
	for (argNum = 1; argNum <= argCount(); argNum++)
		for (motor = 1; motor <= 4; motor++)
			if (motors[motor]) {
				hpSetMotor(motor);
				argParamPairListWrite(argNum);
				paramFUpdate(motor);
			}
			
	return errNone;
}

/* CmdFingerGet outputs the given motor parameters. */
int CmdFingerGet(void)
{
	int err, motor, argNum, needSpace;
	unsigned int value;
	motorFlags motors;
	
	checkErr(argParamListParse(FArgs));
	
	parseGetMotors(motors);
	
	for (argNum = 1; argNum <= argCount(); argNum++) {
		printf("\n");
		needSpace = false;
		for (motor = 1; motor <= 4; motor++) {
			if (motors[motor]) {
				hpSetMotor(motor);
				value = argParamListRead(argNum);
				if (needSpace)
					printf(" %d", value);
				else
					printf("%d", value);
				needSpace = true;
			}
		}
	}
	return errNone;
}

/* CmdFingerLoad loads parameters for all/selected motors from the EEPROM. */
int CmdFingerLoad(void)
{
	int motor;
	motorFlags motors;

	checkMaxArgs(0);
	
	parseGetMotors(motors);
	
	for (motor = 1; motor <= 4; motor++)
		if (motors[motor]) {
			hpSetMotor(motor);
			paramFLoad(motor);
			paramFUpdate(motor);
		}
	return errNone;
}

/* CmdFingerSave saves parameters for all/selected motors to the EEPROM. */
int CmdFingerSave(void)
{
	int motor;
	motorFlags motors;

	checkMaxArgs(0);
	
	parseGetMotors(motors);
	
	for (motor = 1; motor <= 4; motor++)
		if (motors[motor]) {
			hpSetMotor(motor);
			paramFSave(motor);
		}
	return errNone;
}

/* CmdFingerDef sets parameters for all/selected motors to their default. */
int CmdFingerDef(void)
{
	int motor;
	motorFlags motors;

	checkMaxArgs(0);
	
	parseGetMotors(motors);
	
	for (motor = 1; motor <= 4; motor++)
		if (motors[motor]) {
			hpSetMotor(motor);
			paramFDefault(motor);
			paramFUpdate(motor);
		}
	return errNone;
}

/* CmdFingerList lists all public parameter names for motors. */
int CmdFingerList(void)
{
	checkMaxArgs(0);
	
	argPrintList(FArgs, false);
	return errNone;
}

/* CmdFingerListV lists all public parameter values for all/selected motors. */
int CmdFingerListV(void)
{
	motorFlags motors;

	checkMaxArgs(0);
	
	parseGetMotors(motors);
	
	argFPrintValues(false, motors);
	return errNone;
}

/* CmdFingerListA lists all public and private parameter names for motors. */
int CmdFingerListA(void)
{
	checkMaxArgs(0);
	
	argPrintList(FArgs, true);
	return errNone;
}

/* CmdFingerListAV lists all public and private parameter values
	for all/selected motors. */
int CmdFingerListAV(void)
{
	motorFlags motors;

	checkMaxArgs(0);
	
	parseGetMotors(motors);
	
	argFPrintValues(true, motors);
	return errNone;
}

/* CmdParamSet sets the given global parameters. */
int CmdParamSet(void)
{
	int err, argNum;
	
	checkErr(argParamPairListParse(PArgs));
	
	for (argNum = 1; argNum <= argCount(); argNum++)
		argParamPairListWrite(argNum);
	paramGUpdate();
	return errNone;
}

/* CmdParamGet outputs the given global parameters. */
int CmdParamGet(void)
{
	int err, argNum;
	unsigned int value;
	
	checkErr(argParamListParse(PArgs));
	
	for (argNum = 1; argNum <= argCount(); argNum++) {
		value = argParamListRead(argNum);
		printf("\n%d", value);
	}
	return errNone;
}

/* CmdParamLoad loads the global parameters from the EEPROM. */
int CmdParamLoad(void)
{	
	checkMaxArgs(0);
	
	paramGLoad();
	paramGUpdate();
	return errNone;
}

/* CmdParamSave saves the global parameters to the EEPROM. */
int CmdParamSave(void)
{
	checkMaxArgs(0);

	paramGSave();
	return errNone;
}

/* CmdParamDef sets the global parameters to their default values. */
int CmdParamDef(void)
{
	checkMaxArgs(0);
	
	paramGDefault();
	paramGUpdate();
	return errNone;
}

/* CmdParamList lists all public global parameter names. */
int CmdParamList(void)
{
	checkMaxArgs(0);
	
	argPrintList(PArgs, false);
	return errNone;
}

/* CmdParamListV lists all public global parameter values. */
int CmdParamListV(void)
{
	checkMaxArgs(0);
	
	argPPrintValues(false);
	return errNone;
}

/* CmdParamListA lists all public and private global parameter names. */
int CmdParamListA(void)
{
	checkMaxArgs(0);
	
	argPrintList(PArgs, true);
	return errNone;
}

/* CmdParamListAV lists all public and private global parameter values. */
int CmdParamListAV(void)
{
	checkMaxArgs(0);
	
	argPPrintValues(true);
	return errNone;
}

/* CmdReset resets the hand software. */
int CmdReset(void)
{
	_asm("	LDX	0FFFEh	\n");		/* !!!! Test this !!!! */
	_asm("	JMP	0,X	\n");
	return errNone;
}

/* CmdVersion prints out version information for the hand software. */
int CmdVersion(void)
{
	printf( "\n\n");
	printf( "\n        BarrettHand Firmware");
	printf( "\n            Version 4.1");
	printf( "\n           December 1999\n");
	printf( "\n    (C) Barrett Technology Inc.");
	printf( "\n         139 Main Street");
	printf( "\n     Cambridge, MA 02142, USA");
	printf( "\n        Tel: (617) 252-9000");
	printf( "\n        Fax: (617) 252-9021");
	return errNone;
}

/* CmdErr prints the meaning of a given error number. If no argument,
	prints meaning of all errors. */
int CmdErr(void)
{
	ErrorType *errPtr;
	int err;

	checkMaxArgs(1);
	
	if (lineWordCount() == 2) {
		checkErr(argIntListParse());
		err = argIntListGet(1);
	} else {
		err = 0xFFFF;
	}
	
	for (errPtr = &ErrList[0]; errPtr->errNum != errNone; errPtr++)
		if (errPtr->errNum & err)
			printf("\n%d: %s", errPtr->errNum, errPtr->help);
	
	return errNone;
}

/* CmdListPublic lists all public commands. */
int CmdListPublic(void)
{
	listCommands(false);
	return errNone;
}

/* CmdListAll lists all commands. */
int CmdListAll(void)
{
	listCommands(true);
	return errNone;
}

/* ===================== Internal utilities ====================== */

#define NUM_LINE  5     /* number of commands per line */

/* listCommands lists commands. If showAll is true, shows all;
	otherwise just shows public commands. */
void listCommands(int showAll)
{
        int i;
	CommandType *ptr;

        printf( "\n============ Commands Available ==============");
        i = 0;
        for (ptr = &CmdList[0];  ptr->name[0]; ptr++) {
		if (ptr->visible || showAll) {
			if ((i % NUM_LINE) == 0)
				printf("\nCMD:   ");
			printf( "%-7s ", ptr->name);
			i++;
		}
                copReset();
        }
        printf( "\n============================================== ");

        printf("\n\n   Run \"HI\" before using any motor command");
        printf( "\n    Type \"?<cmd>\" to see help on a command");
        printf( "\n");
}

/* CmdOpenOrClose opens or closes all/selected motors. */
int CmdOpenOrClose(int direction)
{
	motorFlags motors;

	checkMaxArgs(0);

	parseGetMotors(motors);

	return motorMoveToLimit(motors, direction);
}


/* CmdIncOpenOrClose opens or closes all/selected motors by the given
	amount, or default. */
int CmdIncOpenOrClose(int direction)
{
	motorFlags motors;
	unsigned int distArray[4 + 1];
	int err, motor;
	
	checkMaxArgs(1);
	
	parseGetMotors(motors);
	
	if (lineWordCount() == 2)
		checkErr(argIntListParse());

	for (motor = 1; motor <= 4; motor++)
		if (lineWordCount() == 2)
			distArray[motor] = argIntListGet(1);
		else
			distArray[motor] = paramFRead(motor, kFParamDS);

	return motorMoveRel(motors, direction, distArray);
}

