/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             139 Main Street                             */
/*             Cambridge, MA 02142, USA                    */
/*             Version 4.0        1998                     */
/*             Author: Daniel Griscom                      */
/*             		   Suitable Systems                    */
/*             		   griscom@suitable.com                */
/*             		   www.suitable.com                    */
/***********************************************************/

/* File: hp.c
	This file holds all the routines that access the HCTL-1100 motor
	controller chips (by Hewlett-Packard; hence "hp.c"). Enjoy. */

/* hpSetMotor sets the current motor and its addresses. Writes to any of these
	addresses puts the given data on the chips' data bus, sets the R/W line
	to write, and additionally sets the given chip control line. Reads
	sets the R/W line to write [DTG, 3/29/98: shouldn't this be "read"?],
	sets the given chip control line, and returns
	any data the chip may put on the data bus. Here are the addresses:
	  motor#      ALE     OE      CS
		1       0xF000  0xF001  0xF002
		2       0xF010  0xF011  0xF012
		3       0xF020  0xF021  0xF022
		4       0xF030  0xF031  0xF032
	*/

#include "bhand.h"
#include "timer.h"
#include "cmd.h"
#include "parse.h"
#include "param.h"
#include "utils.h"
#include "motor.h"
#include "hp.h"

#define maxStoppedJitter 4	/* Max position jitter if motor is
					to be considered stopped */

/*====================== Static vars =========================*/

/* This structure allows access to the HCTL-1100 motor controller chips. */
static struct {
	int ALE;                /* Address Latch Enable address */
	int CS;                 /* Chip Select address */
	int OE;                 /* Output Enable address */
	} hpAccess;

static unsigned int lastTime[4 + 1]; /* Last time motor was known to move (in ticks).
					lastTime[0] not used. */
static PositionType lastPos[4 + 1]; /* Last known motor position, used in stopped
					detection. lastPos[0] not used. */

/*====================== Global vars =========================*/

int curMotor;				/* Current motor selected (1-4) */
motorFlags isBoard;			/* Flag: true iff there is a board here */
motorFlags isMotor;			/* Flag: true iff there is a motor AND board */
motorFlags motorInit;		/* Flag: true iff motor has been initialized */

/*======================= Private prototypes ================*/

int flagsGetIndex(int flag);

/*====================== Routines ===========================*/

/* hpReset resets all the controllers, and clears the isBoard, isMotor and
	motorInit flags. */
void hpReset(void)
{
	int motor;

	/* Put each controller in reset mode. */
	for (motor=1; motor <=4; motor++) {
		hpSetMotor(motor);
		hpModeReset();
		isBoard[motor] = false;
		isMotor[motor] = false;
		motorInit[motor] = false;
	}
}

/* hpSetMotor sets the current motor. */
void hpSetMotor(int motor)
{
	int motorAdd;					/* Base address of motor controller. */
	curMotor = motor;				/* Current motor number (1-4). */
	motorAdd = ((motor - 1) << 4) + 0xF000;      /* Calc base address */

	hpAccess.ALE = motorAdd;        /* Write to this address latches address */
	hpAccess.OE = motorAdd + 1;     /* Read from this address gets data */
	hpAccess.CS = motorAdd + 2;     /* Read/write to this sets chip select;
										write also writes to addressed chip */
	return;
}

/* hpGetMotor returns the current motor. */
int hpGetMotor(void)
{
	return curMotor;
}

/* hpInitChip initializes the current motor controller chip. If can't find 
	a chip, clears isBoard[theChip] and isMotor[theChip]. */
void hpInitChip(void)
{
	timer myTimer;

	hpRegWrite(PROGRAM, PROG_RESET);	/* software reset */

	/* Wait for motor to go into idle mode */
	timerSet(&myTimer, 50);
	while (!timerOver(&myTimer)) {
		copReset();
		if (hpRegRead(FLAGS) & FLAG_IDLE) /* If idle bit set */
			break;
	}

	D2(printf( "\n [%d]th HCTL-1100 controller reset	...", curMotor));
	
	hpFlagClearAll();

	/* Test to see if board is present, by setting and reading register */
	hpRegWrite(COMMAND, 0xA5);
	/* If we don't get the same value back, no board or motor. */
	if (hpRegRead(COMMAND) != 0xA5) {
		isBoard[curMotor] = false;
		isMotor[curMotor] = false;
		return;
	} else
		hpRegWrite(COMMAND, 0x00);
		
	hpRegWrite(SAMPLE, 0x10);			/* set sample timer for 21*16/1E16 ? */
	hpRegWrite(RING,30);				/* 3 phase motor */
	hpRegWrite(OFFSET, 0x00);			/* clear offset register*/
	hpRegWrite(PHASE, 0x00);			/* Phase_store (better value????) */
	hpRegWrite(M_PHASE, 0x30);			/* Max phase advance (better value????)*/
	hpRegWrite(Y, 5);					/* set up X and Y for overlaps */
	hpRegWrite(X, 5);
	hpRegWrite(STATUS, 0x05);			/* 5: select 3 phase, single mode commutation*/
	
	hpSetFilter(kClose);
	D2(printf( " Done! "));
}

/* hpStatusRead reads the status for the current motor. Status is 0 if
	everything OK; is error if otherwise. Argument is ignored. */
unsigned int hpStatusRead(int garbage)
{
	if (!isBoard[curMotor])
		return errBoardNone;
	if (!isMotor[curMotor])
		return errMotorNone;
	if (!motorInit[curMotor])
		return errMotorNotInit;
	return errNone;
}

/* hpRegRead reads a register in the current motor controller. */
char hpRegRead(int address)
{
	int garbage;
	pokeb(hpAccess.ALE, address);   /* Write and latch register address */
	garbage = peekb(hpAccess.CS);   /* Stimulate proper chip's chip select */
	return peekb(hpAccess.OE);      /* Turn on and read chip's output */
}

/* hpRegWrite writes a value to a register in the current motor controller. */
void hpRegWrite(int address, int data)
{
	pokeb(hpAccess.ALE, address);   /* Write and latch register address */
	pokeb(hpAccess.CS, data);       /* Select and write to proper chip */
	return;
}

/* hpFlagSet sets a particular flag in the HP FLAGS register. Takes one of the
	FLAG_* constants (e.g. FLAG_TRAPEZOID). */
void hpFlagSet(int flag)
{
	int index = flagsGetIndex(flag);
//	if (index == -1)
//		printf("\nhpFlagsSet Error: unknown flag %x", flag);
//	else {
		hpRegWrite(FLAGS, index | 0x08);
//		result = hpRegRead(FLAGS);
//		printf("\nhpFlagSet: flag %2.2x with index %x set: result %2.2x",
//				flag, index, result);
//		if ((result & flag) == 0)
//			printf("\nhpFlagsSet Error: didn't set flag %x", flag);
//	}
	return;
}

/* hpFlagClear clears a particular flag in the HP FLAGS register. Takes one of the
	FLAG_* constants (e.g. FLAG_TRAPEZOID). */
void hpFlagClear(int flag)
{
	int index = flagsGetIndex(flag);
//	if (index == -1)
//		printf("\nhpFlagsClear Error: unknown flag %x", flag);
//	else {
		hpRegWrite(FLAGS, index);
//		result = hpRegRead(FLAGS);
//		printf("\nhpFlagCleared: flag %2.2x with index %x cleared: result %2.2x",
//				flag, index, result);
//		if ((result & flag) != 0)
//			printf("\nhpFlagsClear Error: didn't clear flag %x", flag);
//	}
	return;
}

/* hpFlagClearAll clears all the controller's flags. */
void hpFlagClearAll(void)
{
	int readResult;
	hpFlagClear(FLAG_TRAPEZOID);
	hpFlagClear(FLAG_UNIPOLAR);
	hpFlagClear(FLAG_PROP_VEL);
	hpFlagClear(FLAG_HOLD_COMM);
	hpFlagClear(FLAG_INT_VEL);
//	readResult = hpRegRead(FLAGS);
//	if ((readResult & 0xFE) != 0)
//		D3(printf("\nhpClearFlags error: final value %2.2x!!!", readResult));
//	else
//		D3(printf("\nhpClearFlags called, final value %2.2x", readResult));
}

/* hpClearMotionFlags clears flags F0 (trapezoidal), F3 (proportional) and
	F5 (integral). */
void hpClearMotionFlags(void)
{
	hpFlagClear(FLAG_TRAPEZOID);
	hpFlagClear(FLAG_PROP_VEL);
	hpFlagClear(FLAG_INT_VEL);
}

/* hpReadCmdPos reads the command position into the given position structure. */
void hpReadCmdPos(PositionType *pos)
{
	pos->s.lsb = hpRegRead(COMPOS+2);
	pos->s.midb = hpRegRead(COMPOS+1);
	pos->s.msb = hpRegRead(COMPOS);
	/* Sign-extend */
	pos->s.filler = (char)(pos->s.msb & 0x80 ? 0xFF : 0x00);
	return;
}

/* hpWriteCmdPos writes the command position from the given position structure. */
void hpWriteCmdPos(PositionType *pos)
{
	int flags;
	flags = hpRegRead(FLAGS);
	hpRegWrite(COMPOS,pos->s.msb);
	hpRegWrite(COMPOS+1,pos->s.midb);
	hpRegWrite(COMPOS+2,pos->s.lsb);
	D5(printf("\nhpWriteCmdPos with flags %2.2x, position %8.8lx.",
		flags,  pos->plong));
	return;
}

/* hpReadActPos reads the actual position into the given position structure. */
void hpReadActPos(PositionType *pos)
{
	pos->s.lsb = hpRegRead(ACTPOS+2);        /* The order is important! */
	pos->s.midb = hpRegRead(ACTPOS+1);
	pos->s.msb = hpRegRead(ACTPOS);
	/* Sign-extend */
	pos->s.filler = (char)(pos->s.msb & 0x80 ? 0xFF : 0x00);
	return;
}

/* hpWriteActPos writes the actual position from the given position structure. */
void hpWriteActPos(PositionType *pos)
{
	D4(printf("\nhpWritePos with motor %d and position %8.8lx", curMotor, pos->plong));
	hpRegWrite(SETPOS,pos->s.msb);           /* The order is important! */
	hpRegWrite(SETPOS+1,pos->s.midb);
	hpRegWrite(SETPOS+2,pos->s.lsb);
	return;
}

/* hpReadFinalPos reads the final position into the given position structure. */
void hpReadFinalPos(PositionType *pos)
{
	pos->s.lsb = hpRegRead(FINPOS);		/* Note reversed order! */
	pos->s.midb = hpRegRead(FINPOS+1);
	pos->s.msb = hpRegRead(FINPOS+2);
	/* Sign-extend */
	pos->s.filler = (char)(pos->s.msb & 0x80 ? 0xFF : 0x00);
	return;
}

/* hpWriteFinalPos writes the final position from the given position structure. */
void hpWriteFinalPos(PositionType *pos)
{
	hpRegWrite(FINPOS+2,pos->s.msb);	/* Note reversed order! */
	hpRegWrite(FINPOS+1,pos->s.midb);
	hpRegWrite(FINPOS,pos->s.lsb);
	return;
}

/* hpWriteCmdVel writes the command velocity from the given velocity structure. */
void hpWriteCmdVel(VelocityType *vel)
{
	hpRegWrite(CVELLSB, vel->s.lsb);
	hpRegWrite(CVELMSB, vel->s.msb);
	return;
}

/* hpReadActVel reads the actual velocity into the given velocity structure. */
void hpReadActVel(VelocityType *vel)
{
	vel->v16 = hpRegRead(0x34);
	return;
}

/* hpClearAllPos clears all position registers to 0. */
void hpClearAllPos(void)
{
	PositionType pos;
	D5(printf("\nhpClearAllPos: before..."));
	D5(hpPrintState());
	pos.plong = 0;
	hpWriteCmdPos(&pos);
	hpRegWrite(ACTPOS+1, 0);	/* This clears the actual position */
	/* hpWriteActPos(&pos); */
	hpWriteFinalPos(&pos);
	D5(printf("\n                        ...after"));
	D5(hpPrintState());
	return;
}

/* hpModeIdle puts the controller in idle mode. */
void hpModeIdle(void)
{
	hpRegWrite(PROGRAM, PROG_IDLE);
}

/* hpModeReset puts the controller in reset mode. */
void hpModeReset(void)
{
	hpRegWrite(PROGRAM, PROG_RESET);           /* software reset */
}

/* hpModeControl puts the controller in control mode. */
void hpModeControl(void)
{
	hpRegWrite(PROGRAM, PROG_CONTROL);
}

/* hpHasStoppedInit initalizes the motor stopped code for the current motor. */
void hpHasStoppedInit(void)
{
	lastTime[curMotor] = timerGetTicks();
	hpReadActPos(&(lastPos[curMotor]));
}

/* hpHasStopped returns true iff the motor has been stopped for a
	number of ticks. Should be called often while testing. */
int hpHasStopped(void)
{
	PositionType pos;
	unsigned int howLong;

	hpReadActPos(&pos);
	if (labs(pos.plong - lastPos[curMotor].plong) > maxStoppedJitter) {
		lastPos[curMotor].plong = pos.plong;
		lastTime[curMotor] = timerGetTicks();
		return false;
	}

	howLong = paramFRead(curMotor, kFParamTSTOP) / (1000 / ticksPerSec);
	if (timerGetTicks() - lastTime[curMotor] < howLong)
		return false;
	return true;
}

/* hpSetFilter sets the current motor's parameters based on the values stored
	for that motor. Takes a direction parameter. */
void hpSetFilter(int direction)
{
	D6(int maxVel);
	
	hpRegWrite(SAMPLE, 0x10);
	hpRegWrite(GAINK, (int)paramFRead(curMotor, kFParamFPG));
	hpRegWrite(ZERO, (int)paramFRead(curMotor, kFParamFDZ));
	hpRegWrite(POLE, (int)paramFRead(curMotor, kFParamFIP));
	hpRegWrite(ACCEL, (int)(paramFRead(curMotor, kFParamACCEL) & 0xFF));
	hpRegWrite(ACCEL+1, (int)(paramFRead(curMotor, kFParamACCEL) >> 8 & 0xFF));
	/* Note: whereas the 16-bit Command Velocity register has
		12 bits of integer and 4 bits of fractional value,
		the 8-bit Maximum Velocity register is all integer.
		So, to keep things constant, I'm shifting right 4 bits. */
	if (direction == kOpen)
		hpRegWrite(MAX_VEL, (int)paramFRead(curMotor, kFParamMOV) >> 4);
	else
		hpRegWrite(MAX_VEL, (int)paramFRead(curMotor, kFParamMCV) >> 4);
	D6(maxVel = hpRegRead(MAX_VEL));
	D6(printf("\nhpSetFilter: MAX_VEL now %d.", maxVel));
}

/* hpPrintState is a debugging routine that prints out everything you ever
	needed to know about the current HCTL-1100 chip. */
void hpPrintState(void)
{
	PositionType cmdPos, actPos, finalPos;

	printf("\n==== Motor Controller %d: ======", curMotor);

	hpReadCmdPos(&cmdPos);
	hpReadActPos(&actPos);
	hpReadFinalPos(&finalPos);
	printf("\n  cmdPos = %8.8lx, actPos = %8.8lx, finalPos = %8.8lx",
		cmdPos.plong, actPos.plong, finalPos.plong);
	printf("\n  flags = %2.2x, ", hpRegRead(FLAGS));
	printf("status = %2.2x, ", hpRegRead(STATUS));
	printf("gaink = %2.2x, ", hpRegRead(GAINK));
	printf("pole = %2.2x, ", hpRegRead(POLE));
	printf("zero = %2.2x, ", hpRegRead(ZERO));
	
	printf("\n");
}

/*======================= Private routines ======================*/


/* flagsGetIndex takes a flag and returns the proper index to set/clear that flag. */
int flagsGetIndex(int flag)
{
	switch (flag) {
		case FLAG_TRAPEZOID:
			return FLAG_TRAPEZOID_INDEX;
		case FLAG_UNIPOLAR:
			return FLAG_UNIPOLAR_INDEX;
		case FLAG_PROP_VEL:
			return FLAG_PROP_VEL_INDEX;
		case FLAG_HOLD_COMM:
			return FLAG_HOLD_COMM_INDEX;
		case FLAG_INT_VEL:
			return FLAG_INT_VEL_INDEX;
		default:
			return -1;
	}
}
