/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             139 Main Street                             */
/*             Cambridge, MA 02142, USA                    */
/*             Version 4.0        1998                     */
/*             Author: Daniel Griscom                      */
/*             		   Suitable Systems                    */
/*             		   griscom@suitable.com                */
/*             		   www.suitable.com                    */
/***********************************************************/

/* File: hp.c
	This file holds all the routines that access the HCTL-1100 motor
	controller chips (by Hewlett-Packard; hence "hp.c"). Enjoy. */

#ifndef _hp_h
#define _hp_h

/*====================== Types =========================*/

typedef struct {
	char filler;
	char msb;
	char midb;
	char lsb;
	} ThreeposType;

typedef union  {
	long plong;
	ThreeposType s;
	} PositionType;

typedef struct {
	char msb;
	char lsb;
	} TwovelType;

typedef union {
	short v16; /* BCZ 11/04/99 WAS: unsigned v16; */
	TwovelType s;
	} VelocityType;

typedef int motorFlags[4 + 1];        /* Element 0 isn't used for numbering convenience */

/*============================ Globals ========================*/

extern motorFlags isBoard;      /* Flag: true iff there is a board here */
extern motorFlags isMotor;      /* Flag: true iff there is a motor AND board */
extern motorFlags motorInit;    /* Flag: true iff motor has been initialized */

/*============================ #defines ========================*/

/* HCTL-1100 registers */
#define FLAGS           0x00
#define PROGRAM         0x05
#define STATUS          0x07
#define CTRL_MODE       0x03
#define COMMAND         0x08
#define PWM             0x09

/* These four registers are all 3 bytes; value points to lowest byte in memory.
	This byte is the MSB, except for FINPOS, where it is the LSB (ick!). */
#define ACTPOS          0x12		/* Actual position */
#define FINPOS          0x29		/* Final position */
#define COMPOS          0x0C		/* Command position */
#define SETPOS          0x15		/* Set actual position */

#define SAMPLE          0x0F            /* sample time */
#define CVELLSB		0x23		/* command velocity */
#define CVELMSB		0x24
/* #define VELLSB          0x34  */          /* actual velocity, counts/sample */
/* #define VELMSB          0x35 */ 


#define RING            0x18
#define OFFSET          0x1C
#define PHASE           0x19
#define M_PHASE         0x1F
#define X               0x1A
#define Y               0x1B

#define GAINK           0x22
#define POLE            0x21
#define ZERO            0x20
#define ACCEL           0x26            /* acceleration */
#define MAX_VEL         0x28

/* Values for PROGRAM register */
#define PROG_RESET      0x00
#define PROG_IDLE       0x01
#define PROG_ALIGN      0x02
#define PROG_CONTROL    0x03

/* Bits in FLAG register */
#define FLAG_TRAPEZOID  (1 << 0)        /* F0: Trapezoid control flag */
#define FLAG_IDLE       (1 << 1)        /* F1: Init/idle flag */
#define FLAG_UNIPOLAR   (1 << 2)        /* F2: Unipolar mode flag */
#define FLAG_PROP_VEL   (1 << 3)        /* F3: Proportional velocity flag */
#define FLAG_HOLD_COMM  (1 << 4)        /* F4: Hold commutator flag */
#define FLAG_INT_VEL    (1 << 5)        /* F5: Integral velocity flag */

/* Indexes for setting and clearing FLAG bits */
#define FLAG_TRAPEZOID_INDEX      	0x00		/* F0: Trapezoid control flag */
/* Note: can't set or clear init/idle flag */
#define FLAG_UNIPOLAR_INDEX			0x02		/* F2: Unipolar mode flag */
#define FLAG_PROP_VEL_INDEX			0x03		/* F3: Proportional velocity flag */
#define FLAG_HOLD_COMM_INDEX		0x04		/* F4: Hold commutator flag */
#define FLAG_INT_VEL_INDEX			0x05		/* F5: Integral velocity flag */

/*====================== Prototypes ===========================*/

/* hpReset resets all the controllers, and clears the isBoard, isMotor and
	motorInit flags. */
void hpReset(void);

/* hpStatusRead reads the status for the current motor. Status is 0 if
	everything OK; is error if otherwise. Argument is ignored. */
unsigned int hpStatusRead(int garbage);

/* hpSetMotor sets the current motor. */
void hpSetMotor(int motor);

/* hpInitChip initializes the current motor controller chip. If can't find 
	a chip, clears isBoard[theChip] and isMotor[theChip]. */
void hpInitChip(void);

/* hpGetMotor returns the current motor. */
int hpGetMotor(void);

/* hpRegRead reads a register in the current motor controller. */
char hpRegRead(int address);

/* hpRegWrite writes a value to a register in the current motor controller. */
void hpRegWrite(int address, int data);

/* hpFlagSet sets a particular flag in the HP FLAGS register. Takes one of the
	FLAG_* constants (e.g. FLAG_TRAPEZOID). */
void hpFlagSet(int flag);

/* hpFlagClear clears a particular flag in the HP FLAGS register. Takes one of the
	FLAG_* constants (e.g. FLAG_TRAPEZOID). */
void hpFlagClear(int flag);

/* hpFlagClearAll clears all the controller's flags. */
void hpFlagClearAll(void);

/* hpClearMotionFlags clears flags F0 (trapezoidal), F3 (proportional) and
	F5 (integral). */
void hpClearMotionFlags(void);

/* hpReadCmdPos reads the command position into the given position structure. */
void hpReadCmdPos(PositionType *pos);

/* hpWriteCmdPos writes the command position from the given position structure. */
void hpWriteCmdPos(PositionType *pos);

/* hpReadActPos reads the actual position into the given position structure. */
void hpReadActPos(PositionType *pos);

/* hpWriteActPos writes the actual position from the given position structure. */
void hpWriteActPos(PositionType *pos);

/* hpReadFinalPos reads the final position into the given position structure. */
void hpReadFinalPos(PositionType *pos);

/* hpWriteFinalPos writes the final position from the given position structure. */
void hpWriteFinalPos(PositionType *pos);

/* hpWriteCmdVel writes the command velocity from the given velocity structure. */
void hpWriteCmdVel(VelocityType *vel);

/* hpReadActVel reads the actual velocity into the given velocity structure. */
void hpReadActVel(VelocityType *vel);

/* hpClearAllPos clears all position registers to 0. */
void hpClearAllPos(void);

/* hpModeIdle puts the controller in idle mode. */
void hpModeIdle(void);

/* hpModeReset puts the controller in reset mode. */
void hpModeReset(void);

/* hpModeControl puts the controller in control mode. */
void hpModeControl(void);

/* hpHasStoppedInit initalizes the globals for hpHasStoppedTicks. */
void hpHasStoppedInit(void);

/* hpHasStopped returns true iff the motor has been stopped for a
	number of ticks. Should be called often while testing. */
int hpHasStopped(void);

/* hpSetFilter sets the current motor's parameters based on the values stored
	for that motor. Takes a direction parameter. */
void hpSetFilter(int direction);

/* hpPrintState is a debugging routine that prints out everything you ever
	needed to know about the current HCTL-1100 chip. */
void hpPrintState(void);

#endif /* #ifndef _hp_h */
