/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             139 Main Street                             */
/*             Cambridge, MA 02142, USA                    */
/*             Version 4.0        1998                     */
/*             Author: Daniel Griscom                      */
/*             		   Suitable Systems                    */
/*             		   griscom@suitable.com                */
/*             		   www.suitable.com                    */
/***********************************************************/

/* File: line.c
	This contains all the line collection and word recognition
	routines. */

#include "bhand.h"
#include "serial.h"
#include "line.h"
#include "utils.h"

#define CR (0x0D)
#define BS (0x08)
#define DEL (0x10)

/* ============ Static vars =========== */

/* lineBuf is the line buffer. */
static char lineBuf[bufLen];

/* lineEnd is a pointer to the first empty space in the buffer. */
static char *lineEnd;

/* lineWords is the number of words in the current (complete) line. */
static int lineWords;

/* lineWordPtrs is the array of pointers to words. */
static char *lineWordPtrs[maxWords];

/* lineReady is true iff lineWaiting() has returned true. When true,
	no more characters are read into the line buffer, and the
	lineWords array has been built. */
static int lineReady;

/* ============= Utility prototypes =========== */

/* lineParse builds the lineWords array, terminating each word with a null. */
void lineParse(void);

/* ============== Main routines =============== */

/* lineInit initializes the line handling code. */
void lineInit(void)
{
	lineClear();
}

/* lineCollect collects characters into the line buffer until a
	line has been read, or there are no more characters. */
void lineCollect(void)
{
	char c;

	if (lineReady)
		return;
	while (true) {
		if (serCharsWaiting() == 0)
			break;
		copReset();
		c = serCharGet();
		/* Check for CR */
		if (c == CR) {
			D2(printf("\nlineCollect got a <CR>..."));
			lineParse();
			break;
		}
		/* Check for backspace or DEL. */ /* !!!! Make sure both work !!!! */
		if ((c == BS) || (c == DEL)) {
			if (lineEnd > lineBuf) {
				printf(" \b");
				--lineEnd;
			} else
				printf(" ");
			break;
		}
		*lineEnd++ = c;
		if (lineEnd == lineBuf + bufLen - 1) {
			lineParse();
			break;
		}
	}
}

/* lineDone returns true iff a complete line has been read. */
int lineDone(void)
{
	return lineReady;
}

/* lineWordCount returns the number of words in the current line. ONLY call
	if lineDone returns true. */
int lineWordCount(void)
{
	return lineWords;
}

/* lineWordGet returns a pointer to the given null-terminated word.
	Word numbers are 0-based. ONLY call if lineDone returns true. */
char *lineWordGet(int wordNum)
{
	return lineWordPtrs[wordNum];
}

/* lineClear clears the current line. */
void lineClear(void)
{
	lineReady = false;
	lineEnd = lineBuf;
}

/* ================= Utilities ================== */

/* lineParse builds the lineWords array, terminating each word with a null. */
void lineParse(void)
{
	char *c;

	D2(printf("\nAbout to parse the damn thing."));

	lineWords = 0;

	*lineEnd = '\0';

	/* Trim space characters from end of line */
 	while (*--lineEnd == ' ')
		*lineEnd = '\0';
	
	/* Search for words */
	c = lineBuf;
	while (*c != '\0') {
		D2(printf("\n    c = %x.", c));
		/* Skip over any space characters */
		while (*c == ' ')
			*c++ = '\0';
		/* Mark word */
		lineWordPtrs[lineWords++] = c;
		/* Skip over non-space and non-null */
		while (*c != ' ' && *c != '\0')
			c++;
	}

	/* Check for empty line */
	if (**lineWordPtrs == '\0')
		lineWords = 0;

	lineReady = true;
}



