/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             139 Main Street                             */
/*             Cambridge, MA 02142, USA                    */
/*             Version 4.0        1998                     */
/*             Author: Daniel Griscom                      */
/*             		   Suitable Systems                    */
/*             		   griscom@suitable.com                */
/*             		   www.suitable.com                    */
/***********************************************************/

/* File: mi.c
	This is the motor initialization code. */

#include "bhand.h"
#include "hp.h"
#include "cmd.h"
#include "timer.h"
#include "parse.h"
#include "param.h"
#include "motor.h"
#include "utils.h"
#include "serial.h"
#include "mi.h"

#define STEP_SIZE 20    /* Expected position change for a step */
#define STEP_SLOP 10    /* Max tolerated difference from expected */
#define RING_VAL 30     /* Value of ring buffer (offset can't be >=) */

/* ------------------- Internal prototypes -------------------- */

int alignMotor(int motor);
void stepOn(int theMotor);
void stepSet(int theOffset);
int stepMove(int direction, int howMany);
void stepOff(void);
void doAlign(void);
int doHPAlign(int motor);
void waitStill(void);

/* ------------------- Static variables -------------------- */

static int noMovement;         /* Flag: true iff haven't seen any movement */

/* ------------------- Main routine -------------------- */

/* MotorInit initializes the given motors. */
int MotorInit(motorFlags motors)
{
	int motor, count, errors;

	/* Default: each possible motor and board are present, but
		not initialized. */
	for (motor = 1; motor <= 4; motor++)
		if (motors[motor]) {
			isBoard[motor] = true;
			isMotor[motor] = true;
			motorInit[motor] = false;
			hpSetMotor(motor);
			hpInitChip();      /* This may clear isBoard and isMotor */
		}
		
	/* DTG, 5/11/98: Terminate spread motor power */
	if (motors[4] && isMotor[4]) { /* BCZ, 10/27/99: Iff it will be initialized */
		hpSetMotor(4);
		hpModeIdle();
	}

	/* Go through finger motors 3 times */
	for (count = 0; count < 3; count++)
		/* For each finger motor */
		for (motor = 1; motor <= 3; motor++)
			/* If motor selected, board and motor exist, not yet initted, 
				and no aborts waiting, */
			if (motors[motor] && isMotor[motor]
					&& !motorInit[motor] && !serAbortWaiting())
				/* Try to init: if succeed, mark */
				if (alignMotor(motor)) {
					D2(printf("\nalignMotor succeeded..."));
					motorInit[motor] = true;
					motorMoveOneToLimit(motor, kOpen); /* Open finger */
					hpModeIdle();
					hpClearAllPos();    
					motorStopOne(motor);
				}
	/* Now, make sure the fingers are all open. Try three times: */
	for (count = 0; count < 3; count++)
		/* For each finger motor */
		for (motor = 1; motor <= 3; motor++)
			/* If motor selected, board and motor exist, initialized, 
				and no aborts waiting, */
			if (motors[motor] && isMotor[motor]
					&& motorInit[motor] && !serAbortWaiting()) {
				/* Open motor to limit */
				motorMoveOneToLimit(motor, kOpen); /* Open finger */
				hpModeIdle();
				hpClearAllPos();    
				motorStopOne(motor);
			}
				
	/* Now, align palm */
	if (motors[4] && isMotor[4] && !serAbortWaiting())    {
		D3(printf("\n MotorInit: about to align palm motor..."));
		D3(hpPrintState());
		if ((motorInit[4] = alignMotor(4)) != 0) {
			D3(printf("\n MotorInit: aligned palm motor..."));
			D3(hpPrintState());
			motorMoveOneToLimit(4, kOpen);
			D3(printf("\n MotorInit: moved to limit..."));
			D3(hpPrintState());
			hpModeIdle();
			hpClearAllPos();
			motorStopOne(motor);
			D3(printf("\n MotorInit: stopped..."));
			D3(hpPrintState());
		}
	}
	/* Now, see who was properly initialized... */
	errors = errNone;
	
	/* If hasn't been aborted, then return error if motor not initialized */
	if (!serAbortWaiting())
		for (motor = 1; motor <= 4; motor++)
			if (!motors[motor])	/* If motor not selected */
				continue;
			else if (!isBoard[motor])	/* If no board found */
				errors |= errBoardNone;
			else if (!isMotor[motor])	/* If no motor found */
				errors |= errMotorNone;
			else if (!motorInit[motor]) /* If motor not initialized */
				errors |= errMotorNotInit;
	return errors;
}

/* ------------------- Internal routines -------------------- */

/* alignMotor tries to align the given motor. Returns
	true iff successful. */
int alignMotor(int motor)
{
	/* kick off the selected motor*/
	copReset();

	noMovement = true;
	stepOn(motor);

	stepSet(10);
	if (stepMove(-5, 5+6) >= 4)
		if (stepMove(5, 3) == 3) {
    	                D2(printf("\nFirst try worked."));
			stepOff();
			doHPAlign(motor);
			motorStopOne(motor);
			D2(printf("\nmotorStopOne returned..."));
			return 1;
			}
        D2(printf("\nNope: try again..."));
	if (serAbortWaiting()) {
		stepOff();
		return 0;
	}
	stepSet(10);
	if (stepMove(5, 5+6) >= 4)
		if (stepMove(-5, 1) == 1) {
                        D2(printf("\nSecond try worked."));
			stepOff();
//			D2(printf("\nalignMotor: stepOff returned..."));
//			D2(hpPrintState());
			doHPAlign(motor);
//			D2(printf("\nalignMotor: doHPAlign returned..."));
//			D2(hpPrintState());
			motorStopOne(motor);
//			D2(printf("\nalignMotor: motorStopOne returned..."));
//			D2(hpPrintState());
			return 1;
			}
        D2(printf("\nDidn't work."));
	/* If none of the steps caused movement, then clear isMotor[]. */
	if (noMovement)
		isMotor[motor] = false;
	stepOff();
	return 0;

}

/* stepOn turns on stepping for the given motor. */
void stepOn(int theMotor)
{
	PositionType pos;

	hpSetMotor(theMotor);
	pos.plong = 0x100000;
	hpWriteCmdPos(&pos);                    /* Make sure cmdPos <> actPos */
	pos.plong = 0x200000;
	hpWriteActPos(&pos);

	hpRegWrite(PROGRAM, PROG_CONTROL);      /* position control mode */
	D3(printf("\nstepOn: about to set commutator flag..."));
	hpFlagSet(FLAG_HOLD_COMM);

}

/* stepSet sets the current step position. */
void stepSet(int theOffset)
{
        D2(printf("\nstepSet to offset %d.", theOffset));
	hpRegWrite(OFFSET, theOffset);
	hpHasStoppedInit();
	while (!hpHasStopped() && !serAbortWaiting())
		copReset();
}

/* stepMove steps the magnetic field for the current motor in the
	given direction for the given number of steps. Returns
	count of good steps (consecutive, including the last). */
int stepMove(int direction, int howMany)
{
	int count, curOffset, goodCount;
	long lastPos, diffPos;
	PositionType pos;

	curOffset = hpRegRead(OFFSET);
        D2(printf("\nstepMove starting at offset %d.", curOffset));
	goodCount = 0;
	hpReadActPos(&pos);
	for (count = 0; count < howMany; count++) {
		lastPos = pos.plong;
		curOffset += direction;
		if (curOffset >= RING_VAL)
			curOffset -= RING_VAL;
		else if (curOffset < 0)
			curOffset += RING_VAL;
		hpRegWrite(OFFSET, curOffset);
                D2(printf("\n    stepped to %d, ", curOffset));
                hpHasStoppedInit();
		while (!hpHasStopped()) {
			if (serAbortWaiting())
				return 0;
			copReset();
		}
		hpReadActPos(&pos);
		diffPos = pos.plong - lastPos;
		/* If there was movement, clear noMovement. */
		if (diffPos)
			noMovement = false;
		/* Adjust sign of diffPos */
		if (direction < 0)
			diffPos = - diffPos;
                D2(printf(" cur=%8.8lx, last=%8.8lx, diff=%8.8lx: ",
			pos.plong, lastPos, diffPos));
		if ((diffPos > STEP_SIZE + STEP_SLOP) ||
			(diffPos < STEP_SIZE - STEP_SLOP)) {
			goodCount = 0;
                        D2(printf("bad."));
		} else {
			goodCount += 1;
                        D2(printf("good."));
		}
	}
        D2(printf("\n    Done: goodCount =	%d.", goodCount));
	return goodCount;
}

/* stepOff turns off stepping for the current motor. */
void stepOff(void)
{
	hpFlagClearAll();
	hpModeIdle();                   /* Idle mode */
	while (!(hpRegRead(FLAGS) & FLAG_IDLE))
		copReset();             /* Wait for idle */
	hpClearAllPos();                /* Clear actual and command positions */
//	D3(printf("\nstepOff done, flags now %2.2x", hpRegRead(FLAGS)));
}

/* This returns true if success, false if fail. */
int doHPAlign(int motor)
{
	int flg;
	timer myTimer;

	hpSetMotor(motor);

	hpClearAllPos();          /* clr commanded and actual position regs*/

	hpRegWrite(PROGRAM, PROG_ALIGN);
	copReset();

	/* Tries for 2 seconds. */
	timerSet(&myTimer, 200);
	while (!timerOver(&myTimer)) {
		flg = hpRegRead(FLAGS);
		if (!(flg & FLAG_HOLD_COMM))    /* If Hold Commutator flag is clear */
			return true;
	}
	/* Failed: return failure */
	return false;
}
