/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             139 Main Street                             */
/*             Cambridge, MA 02142, USA                    */
/*             Version 4.0        1998                     */
/*             Author: Daniel Griscom                      */
/*             		   Suitable Systems                    */
/*             		   griscom@suitable.com                */
/*             		   www.suitable.com                    */
/***********************************************************/

/* File: motor.c
	This code contains the actual motor control loops. */


#include "bhand.h"
#include "cmd.h"
#include "param.h"
#include "timer.h"
#include "utils.h"
#include "hp.h"
#include "motor.h"
#include "serial.h"

/* =================== Constants ====================== */

#define GET_GOING_MS (50)		/* The time allowed for motors to
						start moving */

/* =================== Main Functions ====================== */

/* motorStopOne stops one motor. If it's the palm, or if parameter
	FMOTOR is true, leaves the power on and the motor in
	position-holding mode. Otherwise turns off motor power. */
void motorStopOne(int motor)
{
	int oldMotor;
	PositionType pos;

	oldMotor = hpGetMotor();
	hpSetMotor(motor);
	/* If we should leave the motor holding position, */
	if (paramFRead(motor, kFParamHOLD)) {
		hpReadActPos(&pos);
		D4(printf("\nmotorStopOne at position %8.8lx.", pos.plong));
		hpWriteCmdPos(&pos);
		hpWriteFinalPos(&pos);
		hpClearMotionFlags();
		hpModeControl();
		hpFlagSet(FLAG_TRAPEZOID);
		D3A(hpPrintState());
	/* Otherwise, turn off motor power. */
	} else
		hpModeIdle();

	hpSetMotor(oldMotor);
	return;
}

/* motorTerminate turns off all/given motor power, regardless of FMOTOR. */
int motorTerminate(motorFlags motors)
{
	int motor, err;

	err = errNone;
	
	for (motor = 1; motor <= 4; motor++)
		if (motors[motor]) {
			hpSetMotor(motor);
			err |= hpStatusRead(0);
			hpModeIdle();
		}
	
	return err;
}

/* motorMoveOneToLimit takes a motor number and a direction, and drives
	the given motor to the limit. Returns error. */
int motorMoveOneToLimit(int motor, int direction)
{
	motorFlags motors;
	int motorNum;

	for (motorNum = 1; motorNum <= 4; motorNum++)
		motors[motorNum] = (motor == motorNum);

	return motorMoveToLimit(motors, direction);
}

/* motorMoveToLimit takes a motorFlags structure and a direction, and
	drives the given motors to the limit. Returns error. */
int motorMoveToLimit(motorFlags motors, int direction)
{
	int motor, keepGoing, err;
	motorFlags isOn;
	VelocityType vel;

	D3(printf("\motorMoveToLimit: here we go..."));

	err = errNone;

	/* kick off motors */
	for (motor = 1; motor <= 4; motor++) {
		D3(printf("\n    motorFlag[%d] = %d.", motor, motors[motor]));
		if (motors[motor]) {
			isOn[motor] = true;
			hpSetMotor(motor);
			hpSetFilter(direction);
			err |= hpStatusRead(0);
			D3(printf("\n motorMoveToLimit: starting motor %d...", motor));
			D3(hpPrintState());
			hpClearMotionFlags();
			hpModeControl();
			if (direction == kOpen)
				vel.v16 = - paramFRead(motor, kFParamMOV);
			else
				vel.v16 = paramFRead(motor, kFParamMCV);

			hpWriteCmdVel(&vel);
			hpFlagSet(FLAG_PROP_VEL);
			D3(printf("\n          ...started."));
			D3(hpPrintState());
			hpHasStoppedInit();
		} else
			isOn[motor] = false;
	}
	timerMSWait(GET_GOING_MS);
	D3(printf("\nOK. All motors are started. Which ones should we stop?"));
	do {
		keepGoing = false;
		for (motor = 1; motor <= 4; motor++)
			if (isOn[motor]) {
				keepGoing = true;
				hpSetMotor(motor);
				/* If we're closing and the strain limit is reached,
					or the motor has stopped moving,
					or ESC has been hit
					then stop this motor. */
				if (((direction == kClose) && (strainGauge(motor) > paramFRead(motor, kFParamMSG)))
					|| hpHasStopped()
				   	|| serAbortWaiting() ) {
					D3(printf("\n   Time to stop motor %d.", motor));
					motorStopOne(motor);
					isOn[motor] = false;
				}
			}
		copReset();
	} while (keepGoing);
	D3(printf("\nThat's all for now..."));
	return err;
}


/* motorMoveToPos takes a motorFlags and a position array, and
	moves the given motors to the position. Returns error. */
int motorMoveToPos(motorFlags motors, PositionType posArray[4 + 1])
{
	int motor, keepGoing, err;
	motorFlags isOn;
	PositionType pos;

	D3(printf("\motorMoveToPos: here we go..."));

	err = errNone;

	/* kick off motors */
	for (motor = 1; motor <= 4; motor++) {
		D3(printf("\n    motorFlag[%d] = %d.", motor, motors[motor]));
		if (motors[motor]) {
			isOn[motor] = true;
			hpSetMotor(motor);
			hpReadActPos(&pos);
			if (posArray[motor].plong > pos.plong)
				hpSetFilter(kClose);
			else
				hpSetFilter(kOpen);
			err |= hpStatusRead(0);
			D3(printf("\n motorMoveToPos: starting motor %d...", motor));
			/* Get into position control at current position */
			hpModeIdle();
			hpClearMotionFlags();
			hpReadActPos(&pos);
			hpWriteCmdPos(&pos);
			hpModeControl();
			/* Now, set up motion */
			hpWriteFinalPos(&posArray[motor]);
			hpFlagSet(FLAG_TRAPEZOID);
			D3(hpPrintState());
			D3(printf("\n          ...started."));
			hpHasStoppedInit();
		} else
			isOn[motor] = false;
	}
	timerMSWait(GET_GOING_MS);
	D3(printf("\nOK. All motors are started. Which ones should we stop?"));
	do {
		keepGoing = false;
		for (motor = 1; motor <= 4; motor++)
			if (isOn[motor]) {
				keepGoing = true;
				hpSetMotor(motor);
				if ((strainGauge(motor) > paramFRead(motor, kFParamMSG)) ||
				   hpHasStopped() ||
				   serAbortWaiting() ) {
					D3(printf("\n   Time to stop motor %d.", motor));
					D3(hpPrintState());
					hpReadActPos(&pos);
					if (labs(pos.plong - posArray[motor].plong) > paramFRead(motor, kFParamMPE))
						err |= errCouldntReach;
					motorStopOne(motor);
					isOn[motor] = false;
					/* !!!! Check if motor reached position !!!! */
				}
			}
		copReset();
	} while (keepGoing);
	D3(printf("\nThat's all for now..."));
	return err;
}


/* motorMoveRel takes a motorFlags, a direction and a distance, and
	moves the given motors the given distance. Returns error. */
int motorMoveRel(motorFlags motors, int direction, unsigned int distArray[4 +1])
{
	int motor;
	motorFlags isOn;
	PositionType pos, posArray[4 + 1];

	D3(printf("\motorMoveRel: here we go..."));

	/* calc new motor positions */
	for (motor = 1; motor <= 4; motor++)
		if (motors[motor]) {
			isOn[motor] = true;
			hpSetMotor(motor);
			hpReadActPos(&pos);
			if (direction == kOpen)
				pos.plong = pos.plong - distArray[motor];
			else
				pos.plong = pos.plong + distArray[motor];
			if (pos.plong < 0)
				posArray[motor].plong = 0;
			else
				posArray[motor].plong = pos.plong;
		}

	return motorMoveToPos(motors, posArray);
}


/* motorPositionGet returns the given motor's position as an unsigned integer, clipped
	to 0. */
unsigned int motorPositionGet(int motor)
{
	PositionType pos;
	unsigned int posInt;

	hpSetMotor(motor);
	hpReadActPos(&pos);

	if (pos.plong < 0)
		posInt = 0;
	else
		posInt = (unsigned int)pos.plong; /* posInt = ((posInt | pos.s.midb) << 8) | pos.s.lsb; 10/21/99 BZ */
		
	return posInt;
}
