/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             139 Main Street                             */
/*             Cambridge, MA 02142, USA                    */
/*             Version 4.0        1998                     */
/*             Author: Daniel Griscom                      */
/*             		   Suitable Systems                    */
/*             		   griscom@suitable.com                */
/*             		   www.suitable.com                    */
/***********************************************************/

/* File: param.c
	This file deals with storing and retrieving all of the various
	user-settable parameters. */

#include "bhand.h"
#include "hp.h"
#include "serial.h"
#include "cmd.h"
#include "utils.h"
#include "param.h"

/* ==================== Parameter storage defs ==================== */

static int fParamArray[4][kFParamSize];	/* Array is 0-based */
static int gParamArray[kGParamSize];

/* EPROM allocations */

#define eeBase 0xF800			/* Boot program starts at 0xF800 */
#define eeStorage (eeBase + 0x200)	/* Leave 512 bytes for boot program */

#ifdef __MWERKS__

static int eeFParamArray[4][kFParamSize]; /* Array is 0-based */
static int eeGParamArray[kGParamSize];

#else

static int eeFParamArray[4][kFParamSize] @eeStorage; /* Array is 0-based */
static int eeGParamArray[kGParamSize] @(eeStorage + sizeof(eeFParamArray));

#endif /* #ifdef __MWERKS__ */

/* Table of errors */
const ErrorType ErrList[] = {
	{ errBoardNone,		"No motor board found"},
	{ errMotorNone,		"No motor found"},
	{ errMotorNotInit,	"Motor not initialized"},
	{ errCouldntReach,	"Couldn't reach position"},
	{ errBadCommand,	"Unknown command"},
	{ errBadParam,		"Unknown parameter name"},
	{ errBadValue,		"Bad value"},
	{ errReadOnly,		"Tried to write read-only parameter"},
	{ errTimeout,		"Timeout"},
	{ errTooManyArgs,	"Too many arguments for this command"},
	{ errBadHeader,		"Bad loop control block header"},
	{ errNoMotorPrefix,	"Command cannot have motor prefix"},
	{ errNone,			""},
	};

/* ==================== Functions =================== */

/* paramInit initalizes the parameters. */
void paramInit(void)
{
	int motor;

	paramGLoad();
	paramGUpdate();
	for (motor = 1; motor <= 4; motor++) {
		paramFLoad(motor);
		paramFUpdate(motor);
		D2(printf("\nparamInit: updated motor %d, state now", motor));
		D2(hpSetMotor(motor));
		D2(hpPrintState());
		copReset();
	}
}

/* paramGRead reads a given global parameter. */
unsigned int paramGRead(int whichParam)
{
	return gParamArray[whichParam];
}

/* paramGWrite writes a given global parameter. */
void paramGWrite(int whichParam, unsigned int value)
{
	gParamArray[whichParam] = value;
}

/* paramGSave saves the global parameters into the EEPROM. */
void paramGSave(void)
{
	eeCopy(eeGParamArray, gParamArray, sizeof(eeGParamArray));
}

/* paramGLoad loads the global parameters from the EEPROM. If the
	parameter values are bad, sets them to their default values. */
void paramGLoad(void)
{
	memcpy(gParamArray, eeGParamArray, sizeof(gParamArray));
	if (gParamArray[kGParamValid] != validValue)
		paramGDefault();
}

/* paramGDefault sets the global parameters to their default values. */
void paramGDefault(void)
{
	gParamArray[kGParamBAUD] = 96;		/* 9600 baud */
	gParamArray[kGParamValid] = validValue;	/* This stuff is valid */
}

/* paramGUpdate updates any registers etc. after any global parameters
	have been changed. Should run after paramGWrite, paramGLoad or
	paramGDefault. */
void paramGUpdate(void)
{
	serBaudSet(gParamArray[kGParamBAUD]);
}

/* paramFRead reads a given motor parameter for the given motor. */
unsigned int paramFRead(int motor, int whichParam)
{
	unsigned int value;
	
	value = fParamArray[motor - 1][whichParam];
	D2(printf("\n  paramFRead[%d][%d] returned %d.", motor - 1, whichParam, value));
	
	return value;
}

/* paramFWrite writes a given motor parameter for the given motor. */
void paramFWrite(int motor, int whichParam, unsigned int value)
{
	fParamArray[motor - 1][whichParam] = value;
}

/* paramFSave saves the given motor's parameters into the EEPROM. */
void paramFSave(int motor)
{
	eeCopy(eeFParamArray[motor - 1],
		fParamArray[motor - 1],
		sizeof(*eeFParamArray));
}

/* paramFLoad loads the given motor's parameters from the EEPROM. If
	the parameter values are bad, sets them to their default values. */
void paramFLoad(int motor)
{
	memcpy(fParamArray[motor - 1],
		eeFParamArray[motor - 1],
		sizeof(*fParamArray));
	if (fParamArray[motor - 1][kFParamValid] != validValue)
		paramFDefault(motor);
}

/* paramFDefault sets the current motor's parameters to their default values. */
void paramFDefault(int motor)
{
	int *paramArray;

	paramArray = fParamArray[motor - 1];

	/* If we're working with the palm, */
	if (motor == 4) {
		paramArray[kFParamMCV] = 35;	/* Close velocity */
		paramArray[kFParamMOV] = 35;	/* Open velocity */
		paramArray[kFParamDS] = 150;	/* Inc step size */
		paramArray[kFParamMSG] = 0x100;	/* Strain limit (irrelevant in palm) */
		paramArray[kFParamDP] = 150;	/* Desired position */
		paramArray[kFParamFPG] = 100;	/* Proportional gain */
		paramArray[kFParamFIP] = 66;	/* Integral pole */
		paramArray[kFParamFDZ] = 221;	/* Derivative zero */
		paramArray[kFParamHOLD] = 1;	/* Palm should hold */
		paramArray[kFParamSGFLIP] = 0;	/* Do NOT flip strain gauge values (change as of 5/11/98) */
		paramArray[kFParamACCEL] = 1;	/* Acceleration */
		paramArray[kFParamMPE] = 25; 	/* Max diff from desired pos */
		paramArray[kFParamTSTOP] = 30;	/* Motor must be still for 30ms */
	/* Else we're working with a finger */
	} else {
		paramArray[kFParamMCV] = 65;	/* Close velocity */
		paramArray[kFParamMOV] = 55;	/* Open velocity */
		paramArray[kFParamDS] = 1200;	/* Inc step size */
		paramArray[kFParamMSG] = 0x100;	/* Strain limit */
		paramArray[kFParamDP] = 1000;	/* Desired position */
		paramArray[kFParamFPG] = 200;	/* Proportional gain */
		paramArray[kFParamFIP] = 66;	/* Integral pole */
		paramArray[kFParamFDZ] = 221;	/* Derivative zero */
		paramArray[kFParamHOLD] = 0;	/* Fingers shouldn't hold */
		paramArray[kFParamSGFLIP] = 0;	/* Do NOT flip strain gauge values (change as of 5/11/98) */
		paramArray[kFParamACCEL] = 1;	/* Acceleration */
		paramArray[kFParamMPE] = 25; 	/* Max diff from desired pos */
		paramArray[kFParamTSTOP] = 30;	/* Motor must be still for 30ms */
	}
	
	/* In either case... */
	paramArray[kFParamEN] = 1;			/* Motor is enabled */
	paramArray[kFParamLCV] = 1;			/* Loop control block contains velocity */
	paramArray[kFParamLCVC] = 1;		/* Loop control velocity multiplied by 1 */
	paramArray[kFParamLCPG] = 0;		/* Loop control block does not contain PG */
	paramArray[kFParamLFV] = 1;			/* Loop feedback block contains velocity */
	paramArray[kFParamLFVC] = 1;		/* Loop feedback velocity divided by 1 */
	paramArray[kFParamLFS] = 1;			/* Loop feedback block contains strain */
	paramArray[kFParamLFAP] = 1;		/* Loop feedback block contains absolute position */
	paramArray[kFParamLFDP] = 0;		/* Loop feedback block does not contain delta position */
	paramArray[kFParamLFDPC] = 1;		/* Delta position isn't scaled */
	paramArray[kFParamValid] = validValue;	/* This data is valid */
}

/* paramFUpdate updates any registers etc. for the current motor after any
	motor parameters have been changed. Should run after paramFWrite,
	paramFLoad or paramFDefault. */
void paramFUpdate(int motor)
{
	/* Nothing to update... */
}
