/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             139 Main Street                             */
/*             Cambridge, MA 02142, USA                    */
/*             Version 4.0        1998                     */
/*             Author: Daniel Griscom                      */
/*             		   Suitable Systems                    */
/*             		   griscom@suitable.com                */
/*             		   www.suitable.com                    */
/***********************************************************/

/* File: param.h
	This file deals with storing and retrieving all of the various
	user-settable parameters. */

#ifndef _param_h
#define _param_h

/* ==================== Parameter storage defs ==================== */

/* Indices into motor parameter arrays: */
#define kFParamMOV		0	/* Maximum open velocity */
#define kFParamMCV		1	/* Maximum close velocity */
#define kFParamMSG		2	/* Maximum allowable finger closing strain */
#define kFParamDS		3	/* Default step size for IO/IC */
#define kFParamDP		4	/* Default position for M */
#define kFParamLCV		5	/* Flag: loop control block contains velocity */
#define kFParamLCVC		6	/* LCV is multiplied by LCVC to get control velocity */
#define kFParamLCPG		7	/* Flag: loop control block contains proportional gain */
#define kFParamLFV		8	/* Flag: loop feedback block contains velocity */
#define kFParamLFVC		9	/* Velocity is divided by LFVC to get LFV */
#define kFParamLFS		10	/* Flag: loop control block contains strain */
#define kFParamLFAP		11	/* Flag: loop control block contains absolute position */
#define kFParamLFDP		12	/* Flag: loop control block contains delta position */
#define kFParamLFDPC	13	/* Delta position is divided by LFDPC to get LFDP */
#define kFParamFPG		14	/* Proportional gain for motor filter */
#define kFParamFIP		15	/* Integral pole for motor filter */
#define kFParamFDZ		16	/* Derivative zero for motor filter */
#define kFParamACCEL	17	/* Acceleration */
#define kFParamMPE		18	/* Max dist to desired pos before error */
#define kFParamTSTOP	19	/* Time (ms) before motor is considered stopped */
#define kFParamHOLD		20	/* Flag: motor should hold position when stopped */
#define kFParamSGFLIP	21	/* Flag: strain gauge values reversed */
#define kFParamEN		22	/* Flag: this motor is included when no motors specified */
#define kFParamValid	23	/* Location which is set to special value when valid */
#define kFParamSize		24	/* Size of motor parameter array */

/* Indices into global parameter array: */
#define kGParamBAUD		0	/* Baud rate divided by 100 */
#define kGParamValid	1	/* Location which is set to special value when valid */
#define kGParamSize		2	/* Size of global parameter array */

/* DTG, 3/29/98: changed validValue from 0xA9C3 to 0xA9C5 */
#define validValue 0xA9C5	/* Valid value for kGParamValid and kFParamValid */

typedef struct {
	int errNum;		/* Number of this error */
	char help[45];		/* String describing error */
	} ErrorType;

extern const ErrorType ErrList[];

/* ==================== Functions =================== */

/* paramInit initalizes the parameters. */
void paramInit(void);

/* paramGRead reads a given global parameter. */
unsigned int paramGRead(int whichParam);

/* paramGWrite writes a given global parameter. */
void paramGWrite(int whichParam, unsigned int value);

/* paramGSave saves the global parameters into the EEPROM. */
void paramGSave(void);

/* paramGLoad loads the global parameters from the EEPROM. If the
	parameter values are bad, sets them to their default values. */
void paramGLoad(void);

/* paramGDefault sets the global parameters to their default values. */
void paramGDefault(void);

/* paramGUpdate updates any registers etc. after any global parameters
	have been changed. Should run after paramGWrite, paramGLoad or
	paramGDefault. */
void paramGUpdate(void);

/* paramFRead reads a given motor parameter for the given motor. */
unsigned int paramFRead(int motor, int whichParam);

/* paramFWrite writes a given motor parameter for the given motor. */
void paramFWrite(int motor, int whichParam, unsigned int value);

/* paramFSave saves the given motor's parameters into the EEPROM. */
void paramFSave(int motor);

/* paramFLoad loads the given motor's parameters from the EEPROM. If
	the parameter values are bad, sets them to their default values. */
void paramFLoad(int motor);

/* paramFDefault sets the current motor's parameters to their default values. */
void paramFDefault(int motor);

/* paramFUpdate updates any registers etc. for the given motor after any
	motor parameters have been changed. Should run after paramFWrite,
	paramFLoad or paramFDefault. */
void paramFUpdate(int motor);

#endif /* #ifndef _param_h */
