/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             139 Main Street                             */
/*             Cambridge, MA 02142, USA                    */
/*             Version 4.0        1998                     */
/*             Author: Daniel Griscom                      */
/*             		   Suitable Systems                    */
/*             		   griscom@suitable.com                */
/*             		   www.suitable.com                    */
/***********************************************************/

/* File: parse.c
	This code deals with parsing the command word in the command line.
	First, call parseIt with a pointer to the word. Then you can
	call the various access routines to get the results of the
	parsing.

	This code decodes any help or motor prefix in a command. The
	possible prefixes are:

	? - Help: help is printed for this command
	1, 2, 3, 4 - command acts on motor 1, 2, 3 or 4, respectively
	S - command acts on finger spread (motor 4)
	G - command acts on finger grasp (motors 1, 2 and 3)
	*/

#include "bhand.h"
#include "hp.h"
#include "cmd.h"
#include "utils.h"
#include "parse.h"
#include "param.h"

/* =================== Utility prototypes ================== */

/* prefixParse takes a pointer to the null-terminated first word in
	the command line. Reads any help character or prefix characters
	from the beginning of the word; returns a pointer to the first
	character that isn't help or a prefix. */
char *prefixParse(char *firstWord);

/* =================== Static vars ================= */

/* parseMotors stores the flags from the most recent prefix parsing. */
static motorFlags parseMotors;

/* wasMotor is true if the last prefixParse found a motor prefix. */
static int wasMotor;

/* wasHelp is true if the last prefixParse found a help prefix. */
static int wasHelp;

/* cmdPtr is a pointer to the command structure found by the last prefixParse.
	Null if nothing found. */
static CommandType *cmdPtr;

/* =================== Routines ================== */

/* parseIt takes a pointer to the null-terminated first word in the
	command line. Parses word for help or prefix characters, and
	searches for a command with the given name. Returns true iff
	successful. */
int parseIt(char *firstWord)
{

	D2(printf("\nEntering parseIt."));

	firstWord = prefixParse(firstWord);

	D2(printf("\nThe first word is %s.", firstWord));
	D2(printf("\n firstWord located at %x.", &firstWord));

	cmdPtr = &(CmdList[0]);

	/* Loop until find command, or until find empty string for command name */
	while (true)  {
		D2(printf("\n  Doing loop with cmdPtr = %x.", cmdPtr));
		copReset();
		if (strcmp(&cmdPtr->name[0], firstWord) == 0) {
			D2(printf("\n    ... found it."));
			return true;
		}
		if (cmdPtr->name[0] != '\0')
			cmdPtr++;
		else
			break;

	}
	D2(printf("\n    ... didn't find it."));
	cmdPtr = 0;
	return false;
}

/* parseCmd returns a pointer to the command structure for the parsed
	command. Returns null if not found. */
CommandType *parseCmd(void)
{
	return cmdPtr;
}

/* parseWasHelp returns true iff there was a help character. */
int parseWasHelp(void)
{
	return wasHelp;
}

/* parseWasMotor returns true iff there were any motor selection characters. */
int parseWasMotor(void)
{
	return wasMotor;
}

/* parseGetMotors sets the given motorFlags structure to the set of motors
	most recently parsed. */
void parseGetMotors(motorFlags motors)
{
	int motor;

	for (motor = 1; motor <= 4; motor++)
		motors[motor] = parseMotors[motor];
}

/* =================== Utilities ================== */

/* prefixParse takes a pointer to the null-terminated first word in
	the command line. Reads any help character or prefix characters
	from the beginning of the word; returns a pointer to the first
	character that isn't help or a prefix. */
char *prefixParse(char *firstWord)
{
	int motor;

	wasMotor = false;
	wasHelp = false;
	for (motor = 1; motor <= 4; motor++)
		parseMotors[motor] = false;

	while (true)
		switch (*firstWord) {

		case ('?'):		/* ???? Make so only accepts one "?" ???? */
			wasHelp = true;
			firstWord++;
			break;
		case ('1'):
			wasMotor = true;
			parseMotors[1] = true;
			firstWord++;
			break;
		case ('2'):
			wasMotor = true;
			parseMotors[2] = true;
			firstWord++;
			break;
		case ('3'):
			wasMotor = true;
			parseMotors[3] = true;
			firstWord++;
			break;
		case ('4'):
		case ('S'):
			wasMotor = true;
			parseMotors[4] = true;
			firstWord++;
			break;
		case ('G'):
			wasMotor = true;
			parseMotors[1] = true;
			parseMotors[2] = true;
			parseMotors[3] = true;
			firstWord++;
			break;
		default:
			if (!wasMotor)
				for (motor = 1; motor <= 4; motor++)
					if (paramFRead(motor, kFParamEN))
						parseMotors[motor] = true;
			return firstWord;
			break;
		}
}

