/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             139 Main Street                             */
/*             Cambridge, MA 02142, USA                    */
/*             Version 4.0        1998                     */
/*             Author: Daniel Griscom                      */
/*             		   Suitable Systems                    */
/*             		   griscom@suitable.com                */
/*             		   www.suitable.com                    */
/***********************************************************/

/* File: serial.c
	This code deals with interrupt-driven serial input. Note that
	serBaudSet assumes a CPU driven by a 4.9152 MHz crystal. 
	All serial output is done using standard library routines. */

#include "bhand.h"
#include "utils.h"
#include "serial.h"

#define serBufLen       (128)
#define ABORT           (0x03)		/* Control-C */
#define CR		(0x0D)
#define sciVec          ((void *)0xffd6)

/* ================== Static variables ================= */

volatile static unsigned char serBuffer[serBufLen ];	/* Buffer for incoming characters */

volatile static char *serBufIn;				/* Pointer to next empty char in buffer */

volatile static  char *serBufOut;			/* Pointer to next char to be read from buffer */

volatile static int serAborts;				/* Number of <ABORT> characters in buffer */

/* ================== Prototypes ================= */

/* serHandler handles the serial interrupt. */
#ifdef __MWERKS__
void serHandler(void);
#else
@port void serHandler(void);
#endif

/* serCGet gets a character from the input stream. Doesn't deal with interrupts,
	doesn't deal with ABORTs, and assumes that there's a character waiting. */
int serCGet(void);

/* serCharGetRaw waits for a character from the input buffer and then returns it. */
int serCharGetRaw(void);

/* ================== Routines ================= */

/* serInit initializes the serial code. Assumes interrupts are disabled. Doesn't
	set baud rate. */
void serInit(void)
{
	char garbage;

	/* Initialize buffer pointers and abort count */
	serBufIn = serBuffer;
	serBufOut = serBuffer;
	serAborts = 0;

	/* Install interrupt handler */
	eeVecSet(sciVec, (voidFuncVoidPtr)serHandler);

	/* Set up serial control registers */
	SCCR1 = 0x00;           /* 8 bits, idle line wakeup (will be disabled) */
	SCCR2 = 0x2C;           /* Receiver full interrupt, enable Tx and Rx */

	/* Punt any character in the serial data register */
	garbage = SCDR;
}

/* serBaudSet sets the serial port baud rate to theRate*100. If the value isn't
	possible then it sets the baud rate to 9600 and returns false; otherwise returns true. */
int serBaudSet(unsigned int theRate)
{
	switch (theRate) {
		case 768:
			BAUD = 0x00;
			break;
		case 384:
			BAUD = 0x01;
			break;
		case 192:
			BAUD = 0x02;
			break;
		case 96:
			BAUD = 0x03;
			break;
		case 48:
			BAUD = 0x04;
			break;
		case 24:
			BAUD = 0x05;
			break;
		case 12:
			BAUD = 0x06;
			break;
		case 6:
			BAUD = 0x07;
			break;
		default:
			serBaudSet(96);
			return false;
	}
	return true;
}

/* serAbortWaiting returns the number of <ABORT> characters in the input buffer. */
int serAbortWaiting(void)
{
	return serAborts;
}

/* serAbortClear flushes the input buffer up to and including the most recent
	<ABORT> characters. If no <ABORT> characters, then does nothing. */
void serAbortClear(void)
{
	while (serAborts > 0) {
		intDisable();
		if (serCGet() == ABORT)
			serAborts--;
		intEnable();
	}
}

/* serCharsWaiting returns the number of characters in the input buffer (unless
	any of them are <ABORT> characters, in which case it returns 0). */
int serCharsWaiting(void)
{
	int result;

	intDisable();                   /* Safe vects */

	if (serAborts != 0)             /* Are there aborts in the buffer? */
		result = 0;
	else if (serBufOut > serBufIn)  /* Have we wrapped around? */
		result = (serBufIn + serBufLen) - serBufOut;
	else                            /* Normal state */
		result = serBufIn - serBufOut;

	intEnable();

	return result;
}

/* serCharGet waits for a character from the input buffer. It takes the character,
	upper-cases it, calls serCharPut with it, and returns it. */
int serCharGet(void)
{
	int c = serCharGetRaw();

	if (c >= 'a' && c <= 'z')
		c -= 'a' - 'A';
	if (c != CR)
		serCharPut(c);
	return c;
}

/* serCharGetRaw waits for a character from the input buffer and then returns it. Keeps track
	of ABORT characters, but returns them anyway. */
int serCharGetRaw(void)
{
	int c;
	volatile char *serBufOutCopy, *serBufInCopy;
	
	/* This works around a bug in the COSMIC 3.6 compiler involving volatile comparisons. */
	do {
		serBufOutCopy = serBufOut;
		serBufInCopy = serBufIn;
	} while (serBufIn == serBufOutCopy);

	intDisable();
	c = serCGet();
	intEnable();
	
	if (c == ABORT)
		serAborts--;
	
	return c;
}

/* serSByteGet gets a single signed byte from the input buffer. Waits until
	a byte is available. Ignores abort characters. */
int serSByteGet(void)
{
	int b = serCharGetRaw();
	if (b & 0x80)
		b |= 0xff00;
	return b;
}

/* serUByteGet gets a single unsigned byte from the input buffer. Waits until
	a byte is available.  Ignores abort characters. */
int serUByteGet(void)
{
	return serCharGetRaw();
}

/* serWordPut sends a 16-bit word as two bytes. Waits until the second byte is 
	being sent. */
void serWordPut(int w)
{
	serCharPut((w >> 8) & 0xFF);
	serCharPut(w & 0xFF);
	return;
}

/* serCharPut sends a character. */
void serCharPut(int c)
{
	while (!(SCSR & 0x80))			/* Wait for empty transmitter buffer */
		;
	SCDR = (char)c;					/* Send the character */
	return;
}

/* ================== Utilities ================= */

/* serHandler handles the serial interrupt. */
#ifdef __MWERKS__
void serHandler(void)
#else
@port void serHandler(void)
#endif
{
	char c;
	int bufSpace;

	if (SCSR & 0x20) {				/* If there's a character waiting */
		c = SCDR;					/* Get it. */
		/* Calculate space left in buffer */
		bufSpace = (serBufOut > serBufIn
				? serBufOut - serBufIn - 1
				: (serBufOut + serBufLen) - serBufIn - 1);
		/* If no space, or if 1 space and it isn't an ABORT, exit */
		if (bufSpace == 0 || (bufSpace == 1 && c != ABORT))
			return;
		if (c == ABORT)
			serAborts++;
		*serBufIn++ = c;
		if (serBufIn == (serBuffer + serBufLen))
			serBufIn = serBuffer;
	}
}

/* serCGet gets a character from the input stream. Doesn't deal with interrupts,
	doesn't deal with ABORTs, and assumes that there's a character waiting. */
int serCGet(void)
{
	int c;
	c = *serBufOut++;
	if (serBufOut == (serBuffer + serBufLen))
		serBufOut = serBuffer;
	return c;
}
