/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             139 Main Street                             */
/*             Cambridge, MA 02142, USA                    */
/*             Version 4.0        1998                     */
/*             Author: Daniel Griscom                      */
/*             		   Suitable Systems                    */
/*             		   griscom@suitable.com                */
/*             		   www.suitable.com                    */
/***********************************************************/

/* File: temp.c
	This code deals with reading the Dallas Semiconductor
	DS1602temperature sensor. Note that the data line is on the
	portD bit 2, the clock line is on port D bit 3,	and the
	reset line is on port D	bit 4. */

#include "bhand.h"
#include "timer.h"
#include "utils.h"
#include "temp.h"

/* External functions:
	int tempInit(void)
		This initializes the temperature sensor. Returns true iff
		successful.
	unsigned int tempRead(int garbage)
		This fetches the temperature as a signed integer in tenths of
		a degree C. Min value -550; max value 1250. Resultion is
		actually a half a degree. Argument is ignored.
*/

/* ======================== Utility functions prototypes =========================== */

/* clearReset raises the /RESET line, enabling communications. */
void clearReset(void);

/* setReset lowers the /RESET line, disabling communications. */
void setReset(void);

/* dataOutput sets the data line to be an output. */
void dataOutput(void);

/* dataInput sets the data line to be an input. */
void dataInput(void);

/* dataWrite sets the data output line to a certain state. */
void dataWrite(int theData);

/* dataRead returns the state of the data line. */
int dataRead(void);

/* clockHigh raises the clock line. */
void clockHigh(void);

/* clockLow lowers the clock line. */
void clockLow(void);

/* toggleClock lowers and raises the clock line. Just before it raises the
	clock, it fetches the state of the data line. Returns that state
	after raising the clock. */
int toggleClock(void);

/* writeProtocol sends the given 8-bit value. */
void writeProtocol(int);

/* write9Data sends the given 9-bit data. */
void write9Data(int theData);

/* read9Data reads the given 9-bit data value. Does NOT sign-extend it. */
int read9Data(void);

/* ======================== Main functions ===================== */

/* tempInit initializes	the temperature	sensor.	*/
void tempInit(void)
{
	setReset();
	clockHigh();
	/* Now,	set the	config register	to binary 00000010; this
		turns on CPU use bit and continuous conversion.	*/
	clearReset();
	writeProtocol(0x0C);	/* Write config	register */
	write9Data(0x02);	/* CPU bit set,	1SHOT bit clear	*/
	setReset();
	timerMSWait(50);	/* Wait 50MS so config register can write */
	clearReset();
	writeProtocol(0xEE);	/* Start converting */
	setReset();
}

/* This	fetches	the temperature	as a signed integer in tenths of
	 a degree C. Min value -550; max value 1250.
	Resultion is actually a	half a degree. Argument is ignored. */
unsigned int tempRead(int garbage)
{
	int theTemp;
	clearReset();
	writeProtocol(0xAA);
	theTemp	= read9Data();
	if (theTemp > 0xFF)	/* Sign-extend the value */
		theTemp	|= (-1 ^ 0xFF);
	setReset();
	return theTemp*5;	/* convert half degrees to tenths */
}

/* =================== Private functions ================= */

/* clearReset raises the RESET line. */
void clearReset(void)
{
	DDRD |=	0x10;  /* Set port D bit 4 to be an output*/
	PORTD |= 0x10; /* Set port D bit 4 to output 1 */
}

/* setReset lowers the RESET line. */
void setReset(void)
{
	DDRD |=	0x10; /* Set port D bit	4 to be	an output */
	PORTD &= 0xEF; /* Set port D bit 4 to output 0 */
}

/* dataOutput sets the data line to be an output. */
void dataOutput(void)
{
	DDRD |=	0x04; /* Set port D bit	2 to be	an output.*/
}

/* dataInput sets the data line	to be an input.	*/
void dataInput(void)
{
	DDRD &=	0xFB; /* Set port D bit	2 to be	an input. */
}

/* dataWrite sets the data line	to the given value. Assumes
	the data line is set to be an output. */
void dataWrite(int theData)
{
	if (theData)
		PORTD |= 0x04;
	else
		PORTD &= 0xFB;
}

/* dataRead reads the data line	and returns it.	*/
int dataRead(void)
{
	if (PORTD & 0x04)
		return 1;
	else
		return 0;
}

/* clockHigh raises the	clock line. */
void clockHigh(void)
{
	DDRD |=	0x08;	/* Set port D bit 3 to be an output */
	PORTD |= 0x08;	/* Set port D bit 3 to be 1 */
}

/* clockLow lowers the clock line. */
void clockLow(void)
{
	DDRD |=	0x08;	 /* Set	port D bit 3 to	be an output */
	PORTD &= 0xF7;	 /* Set	port D bit 3 to	be 1 */
}

/* toggleClock toggles the clock line low then high. Before
   raising the	clock line it latches the data line; it	returns	the
   latched value when done.
*/
int toggleClock(void)
{
	int dataVal;

	clockLow();
	dataVal	= dataRead();
	clockHigh();
	copReset();

	return dataVal;
}

/* writeProtocol writes	the given 8-bit	value. */
void writeProtocol(int theData)
{
	int mask;
	dataOutput();
	for (mask = 0x0001; mask != 0x0100; mask = (mask << 1))	{
		dataWrite(theData & mask);
		toggleClock();
	}
}

/* write9Data writes a nine-bit	data word to the temperature sensor. */
void write9Data(int theData)
{
	int mask;
	dataOutput();
	for (mask = 0x0001; mask != 0x0200; mask = (mask << 1) ) {
		dataWrite(theData & mask);
		toggleClock();
	}
}

/* read9Data reads 9 bits of data from the temperature sensor. */
int read9Data(void)
{
	int theValue, mask, bit;
	theValue = 0;
	dataInput();
	for (mask = 0x0001; mask != 0x0200; mask = (mask	<< 1)) {
		bit = toggleClock();
		theValue |= (bit ? mask	: 0);
	}
	return theValue;
}

