/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             139 Main Street                             */
/*             Cambridge, MA 02142, USA                    */
/*             Version 4.0        1998                     */
/*             Author: Daniel Griscom                      */
/*             		   Suitable Systems                    */
/*             		   griscom@suitable.com                */
/*             		   www.suitable.com                    */
/***********************************************************/

/* File: timer.c
	This code deals with the tick timer and various timing
	routines. It is centered around a timer structure, which
	defines an interval by holding a start time and an end time.
	The code uses Output Compare 4 to run a 10ms timer. The
	timer overflows every 11 minutes. The code assumes that
	the bottom 2 bits of TMSK2 are set to binary 01 giving
	a divide-by-4 prescaler for the timer, and that the CPU
	runs on a 4.9152 MHz crystal (the clock given to the
	prescaler is the crystal divided by 4).

	This uses output compares 3 and 4. */

#include "bhand.h"
#include "utils.h"
#include "timer.h"

#define toc4Vec		((void *)0xffe2)
#define countsPerTick	(3072)	/* Timer counts per tick (10ms) */
#define countsPerMS	(307)	/* Actually 307.2, but who's counting... */

/* ================== Static variables ================= */

static unsigned int timerTicks;		/* Current time, in ticks */

/* ================== Utility prototypes ================= */

/* timerHandler handles the OC4 interrupt. */
#ifdef __MWERKS__
void timerHandler(void);
#else
@port void timerHandler(void);
#endif

/* ================== Routines ================= */

/* timerInit initializes the timer code. It assumes that interrupts are
	disabled. */
void timerInit(void)
{
	/* Install interrupt handler */
	eeVecSet(toc4Vec, (voidFuncVoidPtr)timerHandler);

	/* Set up timer registers */
	TOC4 = TCNT + countsPerTick;	/* Set up time for next OC4 */
	TMSK1 |= 0x10;			/* Enable OC4 interrupt */
	TFLG1 = 0x10;			/* Clear OC4 flag */
	
	/* Make sure that TMSK2 is set properly. */
	if ((TMSK2 & 0x03) != 0x01)
		printf("\ntimerInit: TMSK2 isn't the correct value!");
}

/* timerGetTicks returns the current time (in ticks). */
unsigned int timerGetTicks(void)
{
	return timerTicks;
}

/* timerOver takes a timer and returns true iff the current time
	is outside the structure's defined time.
	Note: I've checked this, and it's OK that interrupts aren't
	disabled during it. */
int timerOver(timer *theTimer)
{
	int result;

	/* If startTime > endTime [wraparound] */
	if (theTimer->startTime > theTimer->endTime)
		result = (timerTicks < theTimer->startTime &&
			theTimer->endTime <= timerTicks);
	else
		result = (timerTicks < theTimer->startTime ||
			theTimer->endTime <= timerTicks);

	return result;
}

/* timerSet sets a timer to start now and end howLong ticks in the
	future. */
void timerSet(timer *theTimer, unsigned int howLong)
{
	theTimer->endTime = timerTicks + howLong;
	theTimer->startTime = timerTicks;
}

/* timerTickWait waits a given number of ticks. */
void timerTickWait(unsigned int howLong)
{
	timer theTimer;
	timerSet(&theTimer, howLong);
	while (!timerOver(&theTimer))
		copReset();
}

/* timerSecWait waits a given number of seconds. Doesn't work for more
	than about 10 minutes (600 seconds). */
void timerSecWait(unsigned int howLong)
{
	timerTickWait(howLong * ticksPerSec);
}

/* timerMSWait waits a given number of milliseconds. Uses Output Compare 3. */
void timerMSWait(unsigned int howLong)
{
	TOC3 = TCNT + countsPerMS;	/* Set timer 3 to trigger in 1 ms */
	TFLG1 = 0x20;			/* Clear timer 3 flag */
	while (howLong-- > 0) {		/* For howLong loops... */
		copReset();
		while (!(TFLG1 & 0x20))	/* Wait for timer 3 flag to set */
			;
		TOC3 = TOC3 + countsPerMS; /* Set timer 3 to trigger in 1 ms */
		TFLG1 = 0x20;		/* Clear timer 3 flag */
	}
}

/* ================== Utilities ================= */

/* timerHandler handles the OC4 interrupt. */
#ifdef __MWERKS__
void timerHandler(void)
#else
@port void timerHandler(void)
#endif
{
	TOC4 = TOC4 + countsPerTick;
	timerTicks++;
	TFLG1 = 0x10;			/* Clear OC4 flag */
}
