/***********************************************************/
/*             (C) Barrett Technology Inc.                 */
/*             139 Main Street                             */
/*             Cambridge, MA 02142, USA                    */
/*             Version 4.0        1998                     */
/*             Author: Daniel Griscom                      */
/*             		   Suitable Systems                    */
/*             		   griscom@suitable.com                */
/*             		   www.suitable.com                    */
/***********************************************************/

/* File: loop.h
	This code contains the loop control functions. */

#include "bhand.h"
#include "hp.h"
#include "cmd.h"
#include "param.h"
#include "motor.h"
#include "loop.h"

/* =================== Static vars ================ */

/* This stores the functions to interpret a control block. For each motor,
	there can be 1 [loopSetMotor] + 2 [control values] + 1 [null terminator] pointers. */
voidFuncVoidPtr controlFuncs[4 * (1 + 2) + 1]; /* BCZ 11/12/99 WAS: [4 * (1 + 2 + 1)] */

/* This stores the functions to send a feedback block. For each motor,
	there can be 1 [loopSetMotor] + 4 [feedback values] + 1 [null terminator] pointers. */
voidFuncVoidPtr feedbackFuncs[4 * (1 + 4) + 1]; /* BCZ 11/12/99 WAS: [4 * (1 + 4 + 1)] */

/* This stores the most recently reported motor positions. */
unsigned int lastReportedPos[4 + 1] = {0, 0, 0, 0, 0};

unsigned int gLastTime[4 + 1]; /* Last time motor was known to move (in ticks).
					lastTime[0] not used. */
long gLastPos[4 + 1]; /* Last known motor position, used in stopped
					detection. lastPos[0] not used. */

/* =================== Header definitions ================ */

#define headerFeedControl	'C'
#define headerAckControl	'c'
#define headerFeed			'A'
#define headerAck			'a'
#define headerCancel		0x03
#define headerAckChar		'*'

/* =================== Local Prototypes ================ */

int doLoop(motorFlags motors);
int prepControl(motorFlags motors);
int prepFeedback(motorFlags motors);
int prepLoop(motorFlags motors);
void sendFeedback(void);
void getControl(void);
void sendAck(void);
void stopLoop(motorFlags motors);
voidFuncVoidPtr getSetMotor(int motor);
void loopGetVelocity(void);
void loopGetGain(void);
void loopPutVelocity(void);
void loopPutStrain(void);
void loopPutAbsPos(void);
void loopPutDeltaPos(void);
void setMotor1(void);
void setMotor2(void);
void setMotor3(void);
void setMotor4(void);

/* =================== Main Functions ====================== */

/* CmdLoop does the loop control function. Returns any error. */
int CmdLoop(void)
{
	motorFlags motors;
	
	if (lineWordCount() > 1)
		return errTooManyArgs;
		
	parseGetMotors(motors);
	
	return doLoop(motors);
}

/* This does the actual work. */
int doLoop(motorFlags motors)
{
	char header;
	int loop = true;
	int err = 0;
	
	if (err = prepControl(motors))
		return err;
	if (err = prepFeedback(motors))
		return err;
	if (err = prepLoop(motors)) {
		stopLoop(motors);
		return err;
	}
	
	// Send initial ACK character
	sendAck();
	
	while (loop) {
		header = serCharGetRaw();
		switch (header) {
			case headerFeedControl:
				sendFeedback();
				getControl();
				break;
			case headerAckControl:
				sendAck();
				getControl();
				break;
			case headerFeed:
				sendFeedback();
				break;
			case headerAck:
				sendAck();
				break;
			case headerCancel:
				loop = false;
				break;
			default:
				err = errBadHeader;
				loop = false;
		}
	}
	
	stopLoop(motors);
	return err;
}

/* ====================== Helper functions ================= */

/* prepControl prepares the function pointers for interpreting control blocks.
	Returns any error. */
int prepControl(motorFlags motors)
{
	voidFuncVoidPtr *p;
	int motor;
	int err = 0;
	
	p = controlFuncs;
	for (motor = 1; motor <= 4; motor++)
		if (motors[motor]) {
			hpSetMotor(motor);
			err |= hpStatusRead(0);
			*p++ = getSetMotor(motor);
			if (paramFRead(motor, kFParamLCV))
				*p++ = loopGetVelocity;
			if (paramFRead(motor, kFParamLCPG))
				*p++ = loopGetGain;
		}
	*p++ = NULL;
	
	return err;
}

/* prepFeedback prepares the function pointers for sending feedback blocks. 
	Returns any error. */
int prepFeedback(motorFlags motors)
{
	voidFuncVoidPtr *p;
	int motor;
	int err = 0;
	
	p = feedbackFuncs;
	for (motor = 1; motor <= 4; motor++)
		if (motors[motor]) {
			hpSetMotor(motor);
			err |= hpStatusRead(0);
			*p++ = getSetMotor(motor);
			if (paramFRead(motor, kFParamLFV))
				*p++ = loopPutVelocity;
			if (paramFRead(motor, kFParamLFS))
				*p++ = loopPutStrain;
			if (paramFRead(motor, kFParamLFAP))
				*p++ = loopPutAbsPos;
			if (paramFRead(motor, kFParamLFDP))
				*p++ = loopPutDeltaPos;
		}
	*p++ = NULL;
	
	return err;
}

/* prepLoop prepares the motor controller(s) for loop control. */
int prepLoop(motorFlags motors)
{
	VelocityType vel;
	int err = 0;
	int motor;
	vel.v16 = 0;
	
	for (motor = 1; motor <= 4; motor++)
		if (motors[motor]) {
			hpSetMotor(motor);
			hpSetFilter(kClose);
			err |= hpStatusRead(0);
			hpClearMotionFlags();
			hpModeControl();
			hpWriteCmdVel(&vel);
			hpFlagSet(FLAG_PROP_VEL);
			lastReportedPos[motor] = motorPositionGet(motor); /* BCZ 11/04/99 */
		}
		
	return err;
			
}

/* sendFeedback sends a feedback block. */
void sendFeedback(void)
{
	voidFuncVoidPtr *p;
	
	sendAck();
	for (p = feedbackFuncs; *p != NULL; p++)
		(*p)();
	return;
}

/* getControl interprets a control block. */
void getControl(void)
{
	voidFuncVoidPtr *p;
	
	for (p = controlFuncs; *p != NULL; p++)
		(*p)();
	return;
}

/* sendAck sends an acknowledgement character. */
void sendAck()
{
	serCharPut(headerAckChar);
	return;
}

/* stopLoop returns the hand to its normal mode. */
void stopLoop(motorFlags motors)
{
	int motor;
	
	for (motor = 1; motor <= 4; motor++)
		if (motors[motor])
			motorStopOne(motor);
	return;
}

/* getSetMotor takes a motor number and returns the proper function to set
	that motor to be the current motor. */
voidFuncVoidPtr getSetMotor(int motor)
{
	switch (motor) {
		case 1:
			return setMotor1;
		case 2:
			return setMotor2;
		case 3:
			return setMotor3;
		case 4:
			return setMotor4;
	}
}

/* loopGetVelocity takes a signed byte of velocity from the input stream and writes
	it to the current motor. */
void loopGetVelocity(void)
{
	VelocityType vel;
	vel.v16 = serSByteGet();
	hpWriteCmdVel(&vel);
	return;
}

/* loopGetGain takes an unsigned byte of proportional gain from the input stream and
	writes it to the current motor. */
void loopGetGain(void)
{
	int gain;
	gain = serUByteGet();
	hpRegWrite(GAINK, gain);
	return;
}

/* loopPutVelocity reads the current velocity, divides it by parameter LFVC, and
	sends it as a signed byte. */
void loopPutVelocity(void)
{
	unsigned int myTime, myPosition;
	VelocityType vel;
	int motor;
	char temp;

	motor = hpGetMotor();

	myTime = timerGetTicks();
	myPosition = motorPositionGet(motor);

	temp = ((myPosition - gLastPos[motor]) / (myTime - gLastTime[motor])) / 3.2;
	
	/* vel.v16 = vel.v16 / 1.67; */

	/* hpReadActVel(&vel); */
	serCharPut(temp / paramFRead(motor, kFParamLFVC)); 

	gLastTime[motor] = myTime;
	gLastPos[motor] = myPosition;
	return;
}	

/* loopPutStrain sends the current motor's strain as an unsigned byte. */
void loopPutStrain(void)
{
	serCharPut(strainGauge(hpGetMotor()));
	return;
}


void loopPutAbsPos(void)
{
	serWordPut(motorPositionGet(hpGetMotor()));
	return;
}

void loopPutDeltaPos(void)
{
	int motor, diff, scaledDiff;
	int scale;
	unsigned int curPos;
	
	motor = hpGetMotor();
	curPos = motorPositionGet(motor);	// Note! this is clipped to zero!
	
	/* Get change since last reported position */
	diff = (int)(curPos - lastReportedPos[motor]);
	
	/* Get scaled diff */
	scale = paramFRead(motor, kFParamLFDPC);
	scaledDiff = diff / scale;
	
	/* Clip to a signed 8-bit value */
	if (scaledDiff > 127)
		scaledDiff = 127;
	else if (scaledDiff < -128)
		scaledDiff = -128;
	
	/* Report scaled value (trimmed to 8 bits) */
	serCharPut(scaledDiff & 0xFF);
	
	/* Update last reported position (remembering to un-scale!) */
	lastReportedPos[motor] = curPos; /* += (scaledDiff * scale); */
	return;
}

/* setMotor1 through setMotor4 sets the current motor to 1 through 4. */
void setMotor1(void)
{
	hpSetMotor(1);
	return;
}

void setMotor2(void)
{
	hpSetMotor(2);
	return;
}

void setMotor3(void)
{
	hpSetMotor(3);
	return;
}

void setMotor4(void)
{
	hpSetMotor(4);
	return;
}
